 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */

import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.util.regex.*;

/*
 * This class contains config informations and utility methods
 *
 */
 
public class Config {
	public static String applicationDir=null;
	public static String unitexDir=null;
	public static String userDir=null;
	public static String currentLanguage=null;
	public static String userCurrentLanguageDir=null;
	public static String unitexCurrentLanguageDir=null;
	public static String currentCorpusDir=null;
	public static String currentGraphDir=null;
	public static String currentSnt=null;
	public static String currentSntDir=null;
	public static String alphabet=null;
	public final static int WINDOWS_SYSTEM=0;
	public final static int LINUX_SYSTEM=1;
	public final static int MAC_OS_X_SYSTEM=2;
	public static String currentSystemName=null;
	public static String currentSentenceGraph=null;
	public static String currentReplaceGraph=null;
	public static int currentSystem=WINDOWS_SYSTEM;
	public static String userName;
	public static String userFile;
	public static String currentDELA;
		
	public static final int MAX_FILE_SIZE=1000000;
	public static final String FILE_TOO_LARGE_MESSAGE="This file is too large to be displayed. Use a wordprocessor to view it.";
	public static final String EMPTY_FILE_MESSAGE="This file is empty.";
	public static final String ERROR_WHILE_READING_FILE_MESSAGE="Cannot read this file.";
	


	public static void initConfig() {
		determineWhichSystemIsRunning();
		determineUnitexDir();		
		determineCurrentUser();
		chooseInitialLanguage();
		setDefaultPreprocessingGraphs();
	}

	public static void determineUnitexDir() {
		String s=System.getProperty("java.class.path");
		if (s.equals(".") || s.equals("Unitex.jar")) {
			setApplicationDir(System.getProperty("user.dir"));
			return;
		}
		int n=s.lastIndexOf(File.separatorChar);
		String path;
		if (n!=-1) path=s.substring(0,n);
		else path=s;
		setApplicationDir(path);
	}

	public static void determineCurrentUser() {
		userName=System.getProperty("user.name");
		if (currentSystem==WINDOWS_SYSTEM) {
			// configuration procedure under Windows
			userFile=getUnitexDir()+"Users"+File.separator+userName+".cfg";
			File f=new File(userFile);
			if (!f.exists()) {
				try {
					chooseNewUserDir();
					f.createNewFile();
					BufferedWriter bw=new BufferedWriter(new FileWriter(f));
					bw.write(getUserDir(),0,getUserDir().length());
					bw.close();
				} catch (IOException e) {e.printStackTrace();}
			}
			else {
				try {
					BufferedReader br=new BufferedReader(new FileReader(f));
					String s=br.readLine();
					if (s==null || s.equals("")) {
						System.out.println("Error: "+userFile+" is empty!");
						s=null;
					}
					setUserDir(s);
					br.close();
				} catch (IOException e) {System.out.println("Error: "+userFile+" is empty!"); e.printStackTrace();}
			}
			return;
		}
		if ((currentSystem==LINUX_SYSTEM) || (currentSystem==MAC_OS_X_SYSTEM)) {
			String rep=System.getProperty("user.home");
			if (!rep.endsWith(File.separator)) rep=rep+File.separator;
			rep=rep+"unitex";
			File f=new File(rep);
			if (!f.exists()) {
				// if the directory does not exist, we inform the user
				String message="Welcome "+getUserName()+"!\n\n";
				message=message+"Your private Unitex directory where you can\nstore your own data is:\n\n";
				message=message+rep+"\n";
				JOptionPane.showMessageDialog(null,message,"Welcome",JOptionPane.PLAIN_MESSAGE); 
			}
			setUserDir(rep);
		}
	}

	public static void determineWhichSystemIsRunning() {
		currentSystemName=System.getProperty("os.name");
		if (currentSystemName.equalsIgnoreCase("Windows NT") ||
		    currentSystemName.equalsIgnoreCase("Windows 2000") ||
		    currentSystemName.equalsIgnoreCase("Windows 98") ||
		    currentSystemName.equalsIgnoreCase("Windows 95") ||
		    currentSystemName.equalsIgnoreCase("Windows XP") ||
		    currentSystemName.equalsIgnoreCase("Windows ME")) {
			currentSystem=WINDOWS_SYSTEM;
		} 
		else if (currentSystemName.equalsIgnoreCase("linux")) {
			currentSystem=LINUX_SYSTEM;
		}
		else if (currentSystemName.equalsIgnoreCase("mac os x")) {
			currentSystem=MAC_OS_X_SYSTEM;
		}
		else {
			System.out.println("Unitex cannot run under "+currentSystemName);
			System.exit(0);
		}
		System.out.println("Unitex is running under "+currentSystemName);
	}

	public static int getCurrentSystem() {
		return currentSystem;
	}

	public static String getCurrentSystemName() {
		return currentSystemName;
	}

	public static String getApplicationDir() {
		if (applicationDir==null) return new String("");
		else return applicationDir;
	}

	public static void setApplicationDir(String s) {
		applicationDir=s;
		if (applicationDir!=null && !applicationDir.endsWith(File.separator)) {
			applicationDir=applicationDir+File.separator;
		}
		int n=applicationDir.lastIndexOf(File.separatorChar,applicationDir.length()-2);
		String path;
		if (n!=-1) path=s.substring(0,n);
		else path=s;
		setUnitexDir(path);
	}

	public static String getUnitexDir() {
		if (unitexDir==null) return new String("");
		else return unitexDir;
	}

	public static void setUnitexDir(String s) {
		unitexDir=s;
		if (unitexDir!=null && !unitexDir.endsWith(File.separator)) {
			unitexDir=unitexDir+File.separator;
		}
		if (!getCurrentLanguage().equals("")) {
			setUnitexCurrentLanguageDir(getUnitexDir()+getCurrentLanguage());
		}
	}

	public static String getUserDir() {
		if (userDir==null) return new String("");
		else return userDir;
	}

	public static void setUserDir(String s) {
		userDir=s;
		if (userDir!=null && !userDir.endsWith(File.separator)) {
			userDir=userDir+File.separator;
		}
		File f=new File(userDir);
		if (!f.exists()) {
			// if the directory does not exists, we create it
			f.mkdir();
		}
		if (!getCurrentLanguage().equals("")) {
			setUserCurrentLanguageDir(getUserDir()+getCurrentLanguage());
		}
	}

	public static String getCurrentLanguage() {
		if (currentLanguage==null) return new String("");
		else return currentLanguage;
	}

	public static void setCurrentLanguage(String s) {
		currentLanguage=s;
		setUnitexCurrentLanguageDir(getUnitexDir()+getCurrentLanguage());
		setUserCurrentLanguageDir(getUserDir()+getCurrentLanguage());
		setCurrentCorpusDir(getUserCurrentLanguageDir()+"Corpus");
		setCurrentGraphDir(getUserCurrentLanguageDir()+"Graphs");
		setAlphabet(getUserCurrentLanguageDir()+"Alphabet.txt");
		setDefaultPreprocessingGraphs();
	}

	public static String getUnitexCurrentLanguageDir() {
		if (unitexCurrentLanguageDir==null) return new String("");
		else return unitexCurrentLanguageDir;
	}

	public static void setUnitexCurrentLanguageDir(String s) {
		unitexCurrentLanguageDir=s;
		if (unitexCurrentLanguageDir!=null && !unitexCurrentLanguageDir.endsWith(File.separator)) {
			unitexCurrentLanguageDir=unitexCurrentLanguageDir+File.separator;
		}
	}

	public static String getUserCurrentLanguageDir() {
		if (userCurrentLanguageDir==null) return new String("");
		else return userCurrentLanguageDir;
	}

	public static void setUserCurrentLanguageDir(String s) {
		userCurrentLanguageDir=s;
		if (userCurrentLanguageDir==null) return;
		if (!userCurrentLanguageDir.endsWith(File.separator)) {
			userCurrentLanguageDir=userCurrentLanguageDir+File.separator;
		}
		// now, we verify if the directory exists
		File f;
		f=new File(userCurrentLanguageDir);
		if (!f.exists()) {
			// if the path does not exists
			f.mkdir();
			copyFileByName(getUnitexCurrentLanguageDir()+"*",getUserCurrentLanguageDir());
		}
		// if any of the sub-directories does not exist, we create it
		f=new File(userCurrentLanguageDir+"Corpus");
		if (!f.exists()) {
			f.mkdir();
			copyFileByName(getUnitexCurrentLanguageDir()+"Corpus"+File.separator+"*",
			               userCurrentLanguageDir+"Corpus"+File.separator);
		}
		f=new File(userCurrentLanguageDir+"Disamb");
		if (!f.exists()) {
			f.mkdir();
			copyFileByName(getUnitexCurrentLanguageDir()+"Disamb"+File.separator+"*",
			               userCurrentLanguageDir+"Disamb"+File.separator);
		}
		f=new File(userCurrentLanguageDir+"Inflection");
		if (!f.exists()) {
			f.mkdir();
			copyFileByName(getUnitexCurrentLanguageDir()+"Inflection"+File.separator+"*",
			               userCurrentLanguageDir+"Inflection"+File.separator);
		}
		f=new File(userCurrentLanguageDir+"Dela");
		if (!f.exists()) {
			f.mkdir();
		}
		f=new File(userCurrentLanguageDir+"Graphs");
		if (!f.exists()) {
			f.mkdir();
		}
		f=new File(userCurrentLanguageDir+"Graphs"+File.separator+"Normalization");
		if (!f.exists()) {
			f.mkdir();
			copyFileByName(getUnitexCurrentLanguageDir()+"Graphs"+File.separator+"Normalization"+File.separator+"*",
			               userCurrentLanguageDir+"Graphs"+File.separator+"Normalization"+File.separator);
		}
		f=new File(userCurrentLanguageDir+"Graphs"+File.separator+"Preprocessing");
		if (!f.exists()) {
			f.mkdir();
		}
		f=new File(userCurrentLanguageDir+"Graphs"+File.separator+"Preprocessing"+File.separator+"Sentence");
		if (!f.exists()) {
			f.mkdir();
			copyFileByName(getUnitexCurrentLanguageDir()+"Graphs"+File.separator+"Preprocessing"+File.separator+"Sentence"+File.separator+"*",
			               userCurrentLanguageDir+"Graphs"+File.separator+"Preprocessing"+File.separator+"Sentence"+File.separator);
		}
		f=new File(userCurrentLanguageDir+"Graphs"+File.separator+"Preprocessing"+File.separator+"Replace");
		if (!f.exists()) {
			f.mkdir();
			copyFileByName(getUnitexCurrentLanguageDir()+"Graphs"+File.separator+"Preprocessing"+File.separator+"Replace"+File.separator+"*",
			               userCurrentLanguageDir+"Graphs"+File.separator+"Preprocessing"+File.separator+"Replace"+File.separator);
		}
	}

	public static String getCurrentCorpusDir() {
		if (currentCorpusDir==null) return new String("");
		else return currentCorpusDir;
	}

	public static void setCurrentCorpusDir(String s) {
		currentCorpusDir=s;
		if (currentCorpusDir!=null && !currentCorpusDir.endsWith(File.separator)) {
			currentCorpusDir=currentCorpusDir+File.separator;
		}
	}

	public static String getCurrentGraphDir() {
		if (currentGraphDir==null) return new String("");
		else return currentGraphDir;
	}

	public static void setCurrentGraphDir(String s) {
		currentGraphDir=s;
		if (currentGraphDir!=null && !currentGraphDir.endsWith(File.separator)) {
			currentGraphDir=currentGraphDir+File.separator;
		}
	}

	public static String getAlphabet() {
		if (alphabet==null) return new String("");
		else return alphabet;
	}

	public static void setAlphabet(String s) {
		alphabet=s;
	}
	
	public static String getUserName() {
		return userName;
	}
	
	
	public static boolean sameDirectory(String a,String b) {
		if (!a.endsWith(File.separator)) a=a+File.separator;
		if (!b.endsWith(File.separator)) b=b+File.separator;
		return a.equalsIgnoreCase(b);
	}
	
	
	public static void chooseNewUserDir() {
		String message="Welcome "+getUserName()+"!\n\n";
		message=message+"To use Unitex, you must choose a private \ndirectory to store your data ";
		message=message+"(that you\ncan change later if you want).";
		message=message+"\n\nClick on OK to choose your directory.";
		JOptionPane.showMessageDialog(null,message,"Welcome",JOptionPane.PLAIN_MESSAGE); 
		JFileChooser f=new JFileChooser();
		f.setDialogTitle("Choose your private directory");
		f.setDialogType(JFileChooser.OPEN_DIALOG);
		f.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		while (f.showOpenDialog(null)!=JFileChooser.APPROVE_OPTION 
		       || sameDirectory(f.getSelectedFile().getAbsolutePath(),getUnitexDir())) {
		       	if (f.showOpenDialog(null)!=JFileChooser.APPROVE_OPTION) {
		       		message="You must choose a private directory.\n\n";
				message=message+"Click on OK to select one or on\n";
				message=message+"Cancel to exit.";
			}
		       	else {
		       		message="You cannot choose the Unitex directory as your private one";
		       	} 
			Object[] options = {"OK","Cancel"};
			int n=JOptionPane.showOptionDialog(null,message,"Error"
			,JOptionPane.YES_NO_OPTION,JOptionPane.ERROR_MESSAGE,null,options,options[0]);
			if (n==1) System.exit(0);
		}
		setUserDir(f.getSelectedFile().getAbsolutePath());
	}
	
	public static void changeUserDir() {
		JFileChooser f=new JFileChooser();
		f.setDialogTitle("Choose your private directory");
		f.setDialogType(JFileChooser.OPEN_DIALOG);
		f.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		if (f.showOpenDialog(null)!=JFileChooser.APPROVE_OPTION) return;
		if (sameDirectory(f.getSelectedFile().getAbsolutePath(),getUnitexDir())) {
	                JOptionPane.showMessageDialog(null,"You cannot choose the Unitex directory as your private one","Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		setUserDir(f.getSelectedFile().getAbsolutePath());
	}
	
	
	public static boolean contains(JComboBox list,String s) {
		if (list==null) return false;
		for (int i=0;i<list.getItemCount();i++) {
			if (s.equals((String)list.getItemAt(i))) return true;
		}
		return false;
		
	}
	
	public static void chooseInitialLanguage() {
		JPanel p=new JPanel();
		p.setLayout(new GridLayout(4,1));
		p.setOpaque(true);
		JComboBox langList=new JComboBox();
		String files_list[]=new File(getUnitexDir()).list();
		for (int i=0;i<files_list.length;i++) {
			if (new File(getUnitexDir()+files_list[i]).isDirectory() && 
				!files_list[i].equals("App") &&
				!files_list[i].equals("Users")) {
				langList.addItem(files_list[i]);
			}
		}
		files_list=new File(getUserDir()).list();
		for (int i=0;i<files_list.length;i++) {
			if (new File(getUserDir()+files_list[i]).isDirectory() && 
				!contains(langList,(String)files_list[i])) {
				langList.addItem(files_list[i]);
			}
		}
		p.add(new JLabel("User: "+getUserName()));
		p.add(new JLabel("Choose the language you want"));
		p.add(new JLabel("to work on:"));
		p.add(langList);
		Object[] options = {"OK","Exit"};
		if (1==JOptionPane.showOptionDialog(null,p,"Unitex"
		,JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,options,options[0])) {
			System.exit(0);
		}
		setCurrentLanguage((String)(langList.getSelectedItem()));
	}



	public static void changeLanguage() {
		JComboBox langList=new JComboBox();
		String files_list[]=new File(getUnitexDir()).list();
		String old=getCurrentLanguage();
		for (int i=0;i<files_list.length;i++) {
			if (new File(getUnitexDir()+files_list[i]).isDirectory() && 
				!files_list[i].equals("App") &&
				!files_list[i].equals("Users"))
			langList.addItem(files_list[i]);
		}
		files_list=new File(getUserDir()).list();
		for (int i=0;i<files_list.length;i++) {
			if (new File(getUserDir()+files_list[i]).isDirectory() && 
				!contains(langList,(String)files_list[i])) {
				langList.addItem(files_list[i]);
			}
		}
		Object[] options = {"OK","Cancel"};
		if (0==JOptionPane.showOptionDialog(null,langList,"Choose a language"
		    ,JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,options,options[0])) {
		    	setCurrentLanguage((String)(langList.getSelectedItem()));
		    	UnitexFrame.mainFrame.setTitle(Version.version+" - current language is "+getCurrentLanguage());
		    	if (!old.equals(getCurrentLanguage())) {
		    		// if the language has really changed
		    		SwingUtilities.invokeLater(new Runnable() {
		    			public void run() {
						UnitexFrame.mainFrame.closeText();
		    				Preferences.setDefaultPreferences();
		    			}
		    		});
		    	}
		}
	}

	public static String getCurrentSnt() {
		return currentSnt;
	}

	public static void setCurrentSnt(String s) {
		currentSnt=s;
		setCurrentSntDir(s);
	}

	public static String getCurrentSntDir() {
		return currentSntDir;
	}

	public static void setCurrentSntDir(String s) {
		currentSntDir=s;
		String path;
		path=s.substring(0,currentSntDir.length()-4);
		currentSntDir=path+"_snt"+File.separator;
	}
	
	public static String getCurrentSentenceGraph() {
		return currentSentenceGraph;
	}

	public static void setCurrentSentenceGraph(String s) {
		currentSentenceGraph=s;
	}

	public static String getCurrentReplaceGraph() {
		return currentReplaceGraph;
	}

	public static void setCurrentReplaceGraph(String s) {
		currentReplaceGraph=s;
	}
	
	public static void setDefaultPreprocessingGraphs() {
		setCurrentSentenceGraph(getUserCurrentLanguageDir()+"Graphs"+File.separator+"Preprocessing"+File.separator+"Sentence"+File.separator+"Sentence.fst2");
		setCurrentReplaceGraph(getUserCurrentLanguageDir()+"Graphs"+File.separator+"Preprocessing"+File.separator+"Replace"+File.separator+"Replace.fst2");
	}
	
	
	public static void copyFileByName(String src,String dest) {
		File b=new File(dest);
		File a=new File(src);
		String path_src=a.getAbsolutePath();
		String expression=path_src.substring(1+path_src.lastIndexOf(File.separatorChar),path_src.length());
		path_src=path_src.substring(0,1+path_src.lastIndexOf(File.separatorChar));
		if (b.isDirectory()) {
			String files_list[]=new File(path_src).list(new RegFileFilter(expression));
			if (files_list!=null) {
			for (int i=0;i<files_list.length;i++) {
				File F;
				if (!(F=new File(path_src+files_list[i])).isDirectory()) {
					copyFile(F,new File(dest+files_list[i]));
				}
			}
			}
		}
		else copyFile(new File(src),b);
	}


	public static void deleteFileByName(String src) {
		File a=new File(src);
		String path_src=a.getAbsolutePath();
		String expression=path_src.substring(1+path_src.lastIndexOf(File.separatorChar),path_src.length());
		path_src=path_src.substring(0,1+path_src.lastIndexOf(File.separatorChar));
		String files_list[]=new File(path_src).list(new RegFileFilter(expression));
		if (files_list!=null) {
			for (int i=0;i<files_list.length;i++) {
				File F=new File(path_src+files_list[i]);
				F.delete();
			}
		}
	}


	/*
	 * This method differs from deleteFileByName because it cannot delete directories
	 */
	public static void removeFile(String src) {
		File a=new File(src);
		String path_src=a.getAbsolutePath();
		String expression=path_src.substring(1+path_src.lastIndexOf(File.separatorChar),path_src.length());
		path_src=path_src.substring(0,1+path_src.lastIndexOf(File.separatorChar));
		String files_list[]=new File(path_src).list(new RegFileFilter(expression));
		for (int i=0;i<files_list.length;i++) {
			File F;
			if (!(F=new File(path_src+files_list[i])).isDirectory()) {
				F.delete();
			}
		}
	}
	
	
	static class RegFileFilter implements FilenameFilter {
		
		public String expression;
		
		RegFileFilter(String exp) {
			expression=exp.replaceAll("\u002C","\\.");
			expression=expression.replaceAll("\\*",".*");
			expression=expression.replaceAll("\\?",".");
		}
		
		public boolean accept(File dir,String f) {
			return Pattern.matches(expression,f);
		}
	}
	
	public static void copyFile(File src,File dest) {
		try {
		FileInputStream fis  = new FileInputStream(src);
		FileOutputStream fos = new FileOutputStream(dest);
		copyStream(fis,fos);
		fis.close();
		fos.close();
		} catch(IOException e) {e.printStackTrace();}
	}

	
	public static void copyStream(InputStream fis,OutputStream fos) {
		try {
		byte[] buf = new byte[2048];
		int i = 0;
		while((i=fis.read(buf))!=-1) {
			fos.write(buf, 0, i);
		}
		} catch(IOException e) {e.printStackTrace();}
	}
	
	
	public static void setCurrentDELA(String s) {
		currentDELA=s;
	}
	
	
	public static String getCurrentDELA() {
		return currentDELA;
	}
	
	
	public static Font getCurrentTextFont() {
		return Preferences.pref.textFont;
	}
}
