 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;


/*
 * GraphBox is the basic object of a graph
 *
 *
 */
class GraphBox {
	
	public int X,Y;    // box coordinates
	public int TYPE;   // 0=initial, 1=final, 2=other
	public boolean variable;
	public boolean selected;    // true is the box is selected
	public boolean comment;     // true is the box is in comment
	public boolean has_out;		// true if any transition goes out this box
	public int has_in;		// true if any transition comes in this box
	public boolean singleDragging;	// true if the box is being moved by the mouse pointer
	public int X_in,Y_in,X_out,Y_out;  // coordinates of the in and out points
	public String contenu;   // box text
	public Vector lines;    // tokenized box lines 
	public Vector greyed;    // greyed[i]=true if the line i is a subgraph call
	public String transduction;  // transduction text, if exists
	public int n_lignes;     // number of visible lines in the box
	public int X1,Y1,Width,Height;  // coordinates of the box frame
	public int h_ligne;  // heigth of a box line
	public int descent;  // heigth of the bottom of letters like jqg
	
	public GraphicalZone parent;
	
	public Vector transitions;
		
	static Font variableFont=new Font("Times New Roman",Font.BOLD,30); 
		
	Graphics context;

	public int numero;	// number used to numerote the state
	
	public GraphBox(int x,int y,int type,Graphics g,GraphicalZone p) {
		X=x; Y=y;
		context=g;
		parent=p;
		TYPE=type;
		contenu="<E>";
		transduction="";
		variable=false;
		n_lignes=0;
		h_ligne=15;
		lines=new Vector();
		greyed=new Vector();
		transitions=new Vector();
		Width=15; Height=20;
		if (TYPE==1) {
			X_in=X; Y_in=Y;
			X1=X; Y1=Y-10;
			Y_out=Y_in;
			X_out=X_in+25;
		} 
		else {
			X1=X; 
			Y1=Y;
			X_in=X;
	    	Y_in=Y/*-Height/2*/;
	    	X_out=X+Width+5;
	    	Y_out=Y_in;
	    }
		selected=false;
		singleDragging=false;
		comment=true;
		has_out=false;
		has_in=0;
		numero=-1;
	}



/*
 * Method to find out if a click is on a sub-graph call
 *
 */
	public String getGraphClicked(int y) {
		int n;
		String s;
		Boolean b;
		n=(y-Y1-4)/(h_ligne);
		if (greyed.isEmpty()) {
			return "";
		}
		b=(Boolean)greyed.get(n);
		if ((!lines.isEmpty()) && b.booleanValue()) {
			s=(String)lines.get(n);
		}
		else {
			s="";
		}
		return s;
	}



/*
 * Methods to handle the box transitions
 *
 */
	public void addTransitionTo(GraphBox g) {
		int i;
		if (this.TYPE==1) {
			// if it is the final, we don't allow parent.pref.output transitions
			return;
		}
		i=transitions.indexOf(g);
		if (i==-1) {
			// if the transition to g does not exist, we create it
			transitions.add(g);
			g.has_in++;
		}
		else {
			// else, we remove it
			transitions.remove(i);
			g.has_in--;
		}
		if (transitions.isEmpty()) 
			has_out=false;
		else
			has_out=true;
		comment=(!has_out && has_in==0);
		g.comment=(!g.has_out && g.has_in==0);
	}







/*
 * Translating method
 *
 */
	public void translate(int dx,int dy) {
		X=X+dx;
		Y=Y+dy;
		X_in=X_in+dx;
		Y_in=Y_in+dy;
		X_out=X_out+dx;
		Y_out=Y_out+dy;
		X1=X1+dx;
		Y1=Y1+dy;
	}



/*
 * Selection detection method
 *
 */
	public boolean isSelectedByRectangle(int Xr,int Yr,int Wr,int Hr) {
		if ( (X1>(Xr+Wr)) || ((X1+Width)<Xr) || (Y1>(Yr+Hr)) || ((Y1+Height)<Yr) ) 
			return false;
		else
			return true;
	}




/*
 * Box drawing methods
 *
 */
	public void drawTransition(Graphics g,GraphBox dest) {
	if (!parent.pref.rightToLeft) {
		if (dest.X_in>this.X_out) {
			// easiest case: drawing a line
			g.drawLine(this.X_out,this.Y_out,dest.X_in,dest.Y_in);
			return;
		}
		if (this.equals(dest)) {
			// if the box is relied to itself
			int diametre1=10+Height/2;
			g.drawArc(X_out-diametre1/2,Y_out-diametre1,diametre1,diametre1,90,-180);
			g.drawArc(dest.X_in-diametre1/2,dest.Y_in-diametre1,diametre1,diametre1,90,180);
			g.drawLine(X_in,Y_in-diametre1,X_out,Y_out-diametre1);
			return;
		}
		if ( (Y1<(dest.Y1+dest.Height)) && ((Y1+Height)>dest.Y1) ) {
			int diametre1=10+Height/2;
			g.drawArc(X_out-diametre1/2,Y_out-diametre1,diametre1,diametre1,90,-180);
			int diametre2=10+dest.Height/2;
			g.drawArc(dest.X_in-diametre2/2,dest.Y_in-diametre2,diametre2,diametre2,90,180);
			int Xpoint1,Ypoint1,Xpoint2,Ypoint2;
			Xpoint1=dest.X_in;
			Ypoint1=dest.Y_in-diametre2;
			Xpoint2=X_out;
			Ypoint2=Y_out-diametre1;
			int Xmilieu,Ymilieu,largeurLimite;
			Xmilieu=(Xpoint2+Xpoint1)/2;
			Ymilieu=(Ypoint2+Ypoint1)/2;
			largeurLimite=diametre1+diametre2;
			Graphics2D g2=(Graphics2D)g;
			QuadCurve2D.Double curve=new QuadCurve2D.Double();
			QuadCurve2D.Double curve2=new QuadCurve2D.Double();
			curve.setCurve(Xpoint1,Ypoint1,
						Xpoint1+largeurLimite,Ypoint1,
						Xmilieu,Ymilieu);
			curve2.setCurve(Xpoint2,Ypoint2,
						Xpoint2-largeurLimite,Ypoint2,
						Xmilieu,Ymilieu);
			g2.draw(curve);
			g2.draw(curve2);
			return;
		}
		if (Y1<(dest.Y1+dest.Height)) {
			int diametre1=10+Height/2;
			g.drawArc(X_out-diametre1/2,Y_out,diametre1,diametre1,0,90);
			int diametre2=10+dest.Height/2;
			g.drawArc(dest.X_in-diametre2/2,dest.Y_in-diametre2,diametre2,diametre2,180,90);
			int Xpoint1,Ypoint1,Xpoint2,Ypoint2;
			Xpoint1=dest.X_in-diametre2/2;
			Ypoint1=dest.Y_in-diametre2/2;
			Xpoint2=X_out+1+diametre1/2;
			Ypoint2=Y_out+1+diametre1/2;
			int hauteurLimite=diametre1+diametre2-20;
			int Xmilieu,Ymilieu;
			Xmilieu=(Xpoint2+Xpoint1)/2;
			Ymilieu=(Ypoint2+Ypoint1)/2;
			Graphics2D g2=(Graphics2D)g;
			QuadCurve2D.Double curve=new QuadCurve2D.Double();
			QuadCurve2D.Double curve2=new QuadCurve2D.Double();
			curve.setCurve(Xpoint1,Ypoint1,
						Xpoint1,Ypoint1-hauteurLimite,
						Xmilieu,Ymilieu);
			curve2.setCurve(Xpoint2,Ypoint2,
						Xpoint2,Ypoint2+hauteurLimite,
						Xmilieu,Ymilieu);
			g2.draw(curve);
			g2.draw(curve2);
			return;
		}
		else {
			int diametre1=10+Height/2;
			g.drawArc(X_out-diametre1/2,Y_out-diametre1,diametre1,diametre1,270,90);
			int diametre2=10+dest.Height/2;
			g.drawArc(dest.X_in-diametre2/2,dest.Y_in,diametre2,diametre2,90,90);
			int Xpoint1,Ypoint1,Xpoint2,Ypoint2;
			Xpoint1=dest.X_in-diametre2/2;
			Ypoint1=dest.Y_in+diametre2/2;
			Xpoint2=X_out+1+diametre1/2;
			Ypoint2=Y_out+1-diametre1/2;
			int hauteurLimite=diametre1+diametre2-20;
			int Xmilieu,Ymilieu;
			Xmilieu=(Xpoint2+Xpoint1)/2;
			Ymilieu=(Ypoint2+Ypoint1)/2;
			Graphics2D g2=(Graphics2D)g;
			QuadCurve2D.Double curve=new QuadCurve2D.Double();
			QuadCurve2D.Double curve2=new QuadCurve2D.Double();
			curve.setCurve(Xpoint1,Ypoint1,
						Xpoint1,Ypoint1+hauteurLimite,
						Xmilieu,Ymilieu);
			curve2.setCurve(Xpoint2,Ypoint2,
						Xpoint2,Ypoint2-hauteurLimite,
						Xmilieu,Ymilieu);
			g2.draw(curve);
			g2.draw(curve2);
			return;
		}
	} 
		// end of the left to right mode
		
	else {
		if (dest.X_out-5<this.X_in-5) {
			// easiest case: drawing a line
			g.drawLine(this.X_in-5,this.Y_in,dest.X_out-5,dest.Y_out);
			return;
		}
		if (this.equals(dest)) {
			// if the box is relied to itself
			int diametre1=10+Height/2;
			g.drawArc(X_in-5-diametre1/2,Y_in-diametre1,diametre1,diametre1,90,180);
			g.drawArc(dest.X_out-5-diametre1/2,dest.Y_out-diametre1,diametre1,diametre1,90,-180);
			g.drawLine(X_out-5,Y_out-diametre1,X_in-5,Y_in-diametre1);
			return;
		}
		if ( (Y1<(dest.Y1+dest.Height)) && ((Y1+Height)>dest.Y1) ) {
			int diametre1=10+Height/2;
			g.drawArc(X_in-5-diametre1/2,Y_in-diametre1,diametre1,diametre1,90,180);
			int diametre2=10+dest.Height/2;
			g.drawArc(dest.X_out-5-diametre2/2,dest.Y_out-diametre2,diametre2,diametre2,90,-180);
			int Xpoint1,Ypoint1,Xpoint2,Ypoint2;
			Xpoint2=dest.X_out-5;
			Ypoint2=dest.Y_out-diametre2;
			Xpoint1=X_in-5;
			Ypoint1=Y_in-diametre1;
			int Xmilieu,Ymilieu,largeurLimite;
			Xmilieu=(Xpoint2+Xpoint1)/2;
			Ymilieu=(Ypoint2+Ypoint1)/2;
			largeurLimite=diametre1+diametre2;
			Graphics2D g2=(Graphics2D)g;
			QuadCurve2D.Double curve=new QuadCurve2D.Double();
			QuadCurve2D.Double curve2=new QuadCurve2D.Double();
			curve.setCurve(Xpoint1,Ypoint1,
						Xpoint1+largeurLimite,Ypoint1,
						Xmilieu,Ymilieu);
			curve2.setCurve(Xpoint2,Ypoint2,
						Xpoint2-largeurLimite,Ypoint2,
						Xmilieu,Ymilieu);
			g2.draw(curve);
			g2.draw(curve2);
			return;
		}
		if (Y1<(dest.Y1+dest.Height)) {
			int diametre1=10+Height/2;
			g.drawArc(X_in-5-diametre1/2,Y_in,diametre1,diametre1,90,90);
			int diametre2=10+dest.Height/2;
			g.drawArc(dest.X_out-5-diametre2/2,dest.Y_out-diametre2,diametre2,diametre2,270,90);
			int Xpoint1,Ypoint1,Xpoint2,Ypoint2;
			Xpoint1=dest.X_out+1-5+diametre2/2;
			Ypoint1=dest.Y_out-diametre2/2;
			Xpoint2=X_in-5-diametre1/2;
			Ypoint2=Y_in+diametre1/2;
			int hauteurLimite=diametre1+diametre2-20;
			int Xmilieu,Ymilieu;
			Xmilieu=(Xpoint2+Xpoint1)/2;
			Ymilieu=(Ypoint2+Ypoint1)/2;
			Graphics2D g2=(Graphics2D)g;
			QuadCurve2D.Double curve=new QuadCurve2D.Double();
			QuadCurve2D.Double curve2=new QuadCurve2D.Double();
			curve.setCurve(Xpoint1,Ypoint1,
						Xpoint1,Ypoint1-hauteurLimite,
						Xmilieu,Ymilieu);
			curve2.setCurve(Xpoint2,Ypoint2,
						Xpoint2,Ypoint2+hauteurLimite,
						Xmilieu,Ymilieu);
			g2.draw(curve);
			g2.draw(curve2);
			return;
		}
		else {
			int diametre1=10+Height/2;
			g.drawArc(X_in-5-diametre1/2,Y_in-diametre1,diametre1,diametre1,270,-90);
			int diametre2=10+dest.Height/2;
			g.drawArc(dest.X_out-5-diametre2/2,dest.Y_out,diametre2,diametre2,90,-90);
			int Xpoint1,Ypoint1,Xpoint2,Ypoint2;
			Xpoint1=dest.X_out-5+diametre2/2;
			Ypoint1=dest.Y_out+1+diametre2/2;
			Xpoint2=X_in-5-diametre1/2;
			Ypoint2=Y_in-diametre1/2;
			int hauteurLimite=diametre1+diametre2-20;
			int Xmilieu,Ymilieu;
			Xmilieu=(Xpoint2+Xpoint1)/2;
			Ymilieu=(Ypoint2+Ypoint1)/2;
			Graphics2D g2=(Graphics2D)g;
			QuadCurve2D.Double curve=new QuadCurve2D.Double();
			QuadCurve2D.Double curve2=new QuadCurve2D.Double();
			curve.setCurve(Xpoint1,Ypoint1,
						Xpoint1,Ypoint1+hauteurLimite,
						Xmilieu,Ymilieu);
			curve2.setCurve(Xpoint2,Ypoint2,
						Xpoint2,Ypoint2-hauteurLimite,
						Xmilieu,Ymilieu);
			g2.draw(curve);
			g2.draw(curve2);
			return;
		}
	}
	}


	public void drawTransitions(Graphics gr) {
		int i,L;
		GraphBox g;
		if (transitions.isEmpty()) return;
		L=transitions.size();
		gr.setColor(parent.pref.foregroundColor);
		for (i=0;i<L;i++) {
			g=(GraphBox)transitions.get(i);
			drawTransition(gr,g);
		}	
	}


  	public void drawOtherSingleDrag(Graphics g) {
		if (comment)
			g.setColor(parent.pref.commentColor);
		else 
			g.setColor(parent.pref.foregroundColor);
		// drawing the box
		if (n_lignes==0) {
			g.drawLine(X_in,Y_in,X_in+15,Y_in);
			if (parent.pref.rightToLeft==false)
				g.drawLine(X_in+15,Y1,X_in+15,Y1+Height);
			else g.drawLine(X_in,Y1,X_in,Y1+Height);
		}
		else {
			g.drawRect(X1,Y1,Width,Height);
		}
		// and the triangle if necessary
		if (has_out || TYPE==0) {
			if (parent.pref.rightToLeft==false) {
				g.drawLine(X_out,Y_out,X1+Width,Y1);
				g.drawLine(X1+Width,Y1,X1+Width,Y1+Height);
				g.drawLine(X1+Width,Y1+Height,X_out,Y_out);
			}
			else {
				g.drawLine(X_in-5,Y_in,X1,Y1);
				g.drawLine(X1,Y1,X1,Y1+Height);
				g.drawLine(X1,Y1+Height,X_in-5,Y_in);
			}
		}
	}


	public void drawInitialSingleDrag(Graphics g) {
		g.setColor(parent.pref.foregroundColor);
		// drawing the box
		if (n_lignes==0) {
			g.drawLine(X_in,Y_in,X_in+15,Y_in);
		}
		else {
			g.drawRect(X1,Y1,Width,Height);
		}
		// drawing the entry line
		if (parent.pref.rightToLeft==false)
			g.drawLine(X_in,Y_in,X_in-10,Y_in);
		else
			g.drawLine(X_out-5,Y_out,X_out+5,Y_out);
		// and the triangle if necessary
		if (has_out || TYPE==0) {
			if (parent.pref.rightToLeft==false) {
				g.drawLine(X_out,Y_out,X1+Width,Y1);
				g.drawLine(X1+Width,Y1,X1+Width,Y1+Height);
				g.drawLine(X1+Width,Y1+Height,X_out,Y_out);
			}
			else {
				g.drawLine(X_in-5,Y_in,X1,Y1);
				g.drawLine(X1,Y1,X1,Y1+Height);
				g.drawLine(X1,Y1+Height,X_in-5,Y_in);
			}
		}
	}


	public void drawFinalSingleDrag(Graphics g) {
		drawFinal(g);
	}


	public void drawOtherComment(Graphics g) {
		int i;
		Boolean is_greyed;
		String l;
		if (variable) {
			drawVariableComment(g);
			return;
		}
		g.setColor(parent.pref.commentColor);
		// print lines if the box is empty
		if (n_lignes==0) {
			g.drawLine(X_in,Y_in,X_in+15,Y_in);
			if (parent.pref.rightToLeft==false)
				g.drawLine(X_in+15,Y1,X_in+15,Y1+Height);
			else g.drawLine(X_in,Y1,X_in,Y1+Height);
		}
		else {
			g.setColor(parent.pref.backgroundColor);
			g.fillRect(X1+1,Y1+1,Width-1,Height-1);
			g.setColor(parent.pref.commentColor);
		}
		// prints the lines of the box
		for (i=0;i<n_lignes;i++) {
		  is_greyed=(Boolean)greyed.get(i);
		  l=(String)lines.get(i);
		  if (is_greyed.booleanValue()) {
		  	g.setColor(parent.pref.subgraphColor);
		  	g.fillRect(X1+3,Y1+4+(i)*h_ligne,Width-4,h_ligne);
		  	g.setColor(parent.pref.commentColor);
		  	g.drawString(l,X1+5,Y1-descent+3+(i+1)*h_ligne);
		  } 
		  else {
			g.drawString(l,X1+5,Y1-descent+3+(i+1)*h_ligne);
		  }
		}
		// prints the transduction, if exists
		g.setColor(parent.pref.foregroundColor);
		if (!transduction.equals("")) {
			g.setFont(parent.pref.output);
			g.drawString(transduction,X1+5,Y1+Height+g.getFontMetrics().getHeight());
		}
	}


	public void drawFinal(Graphics g) {
		g.setColor(parent.pref.backgroundColor);
		g.fillOval(X,Y-10,21,21);
		g.setColor(parent.pref.foregroundColor);
		g.drawOval(X,Y-10,21,21);
		g.drawRect(X+5,Y-5,10,10);
	}
	
	
	public void drawFinalSelected(Graphics g) {
		g.setColor(parent.pref.selectedColor);
		g.fillOval(X,Y-10,21,21);
		g.setColor(parent.pref.backgroundColor);
		g.drawRect(X+5,Y-5,10,10);
	}

	
	public void drawVariable(Graphics g) {
		g.setColor(parent.pref.commentColor);
		g.setFont(variableFont);
		g.drawString((String)lines.get(0),X1+5,Y1-g.getFontMetrics().getDescent()+get_h_variable_ligne());
		g.setFont(parent.pref.output);
		g.drawString(transduction,X1+10,Y1+Height+g.getFontMetrics().getHeight());
	}


	public void drawVariableSelected(Graphics g) {
		g.setColor(parent.pref.selectedColor);
		g.fillRect(X1,Y1,Width,Height);
		g.setColor(parent.pref.commentColor);
		g.setFont(variableFont);
		g.drawString((String)lines.get(0),X1+5,Y1-g.getFontMetrics().getDescent()+get_h_variable_ligne());

		g.setColor(parent.pref.selectedColor);
		g.fillRect(X1+5,Y1+Height+g.getFontMetrics().getDescent()
		           ,context.getFontMetrics(parent.pref.output).stringWidth(transduction)
		           ,context.getFontMetrics(parent.pref.output).getHeight()+1);
		g.setColor(parent.pref.backgroundColor);
		g.setFont(parent.pref.output);
		g.drawString(transduction,X1+5,Y1+Height+g.getFontMetrics().getHeight());
	}


	public void drawVariableComment(Graphics g) {
		drawVariable(g);
	}
	
	
	public void drawOther(Graphics g) {
		int i;
		Boolean is_greyed;
		String l;
		if (variable) {
			drawVariable(g);
			return;
		}
		g.setColor(parent.pref.foregroundColor);
		// drawing the box
		if (n_lignes==0) {
			g.drawLine(X_in,Y_in,X_in+15,Y_in);
			if (parent.pref.rightToLeft==false)
				g.drawLine(X_in+15,Y1,X_in+15,Y1+Height);
			else g.drawLine(X_in,Y1,X_in,Y1+Height);
		}
		else {
			g.drawRect(X1,Y1,Width,Height);
			g.setColor(parent.pref.backgroundColor);
			g.fillRect(X1+1,Y1+1,Width-1,Height-1);
			g.setColor(parent.pref.foregroundColor);
		}
		// and the triangle if necessary
		if (has_out || TYPE==0) {
			if (parent.pref.rightToLeft==false) {
				g.drawLine(X_out,Y_out,X1+Width,Y1);
				g.drawLine(X1+Width,Y1,X1+Width,Y1+Height);
				g.drawLine(X1+Width,Y1+Height,X_out,Y_out);
			}
			else {
				g.drawLine(X_in-5,Y_in,X1,Y1);
				g.drawLine(X1,Y1,X1,Y1+Height);
				g.drawLine(X1,Y1+Height,X_in-5,Y_in);
			}
		}
		// prints the lines of the box
		for (i=0;i<n_lignes;i++) {
		  is_greyed=(Boolean)greyed.get(i);
		  l=(String)lines.get(i);
		  if (is_greyed.booleanValue()) {
		  	g.setColor(parent.pref.subgraphColor);
		  	g.fillRect(X1+3,Y1+4+(i)*h_ligne,Width-4,h_ligne);
		  	g.setColor(parent.pref.foregroundColor);
		  	g.drawString(l,X1+5,Y1-descent+3+(i+1)*h_ligne);
		  } 
		  else {
			g.drawString(l,X1+5,Y1-descent+3+(i+1)*h_ligne);
		  }
		}
		// prints the transduction, if exists
		g.setColor(parent.pref.foregroundColor);
		if (!transduction.equals("")) {
			g.setFont(parent.pref.output);
			g.drawString(transduction,X1+5,Y1+Height+g.getFontMetrics().getHeight());
		}
	}
	
	
	public void drawOtherSelected(Graphics g) {
		int i;
		String l;
		if (variable) {
			drawVariableSelected(g);
			return;
		}
		g.setColor(parent.pref.foregroundColor);
		// drawing the box
		if (n_lignes==0) {
			g.setColor(parent.pref.selectedColor);
			g.fillRect(X_in,Y_in-10,15,20);
			g.setColor(parent.pref.backgroundColor);
			g.drawLine(X_in,Y_in,X_in+15,Y_in);
		}
		else {
			g.setColor(parent.pref.selectedColor);
			g.fillRect(X1,Y1,Width,Height+1);
		}
		// and the triangle if necessary
		if (has_out || TYPE==0) {
			g.setColor(parent.pref.foregroundColor);
			if (parent.pref.rightToLeft==false) {
				g.drawLine(X_out,Y_out,X1+Width,Y1);
				g.drawLine(X1+Width,Y1,X1+Width,Y1+Height);
				g.drawLine(X1+Width,Y1+Height,X_out,Y_out);
			}
			else {
				g.drawLine(X_in-5,Y_in,X1,Y1);
				g.drawLine(X1,Y1,X1,Y1+Height);
				g.drawLine(X1,Y1+Height,X_in-5,Y_in);
			}
		}
		// prints the lines of the box
		g.setColor(parent.pref.backgroundColor);
		for (i=0;i<n_lignes;i++) {
			l=(String)lines.get(i);
			g.drawString(l,X1+5,Y1-descent+3+(i+1)*h_ligne);
		}
		// prints the transduction, if exists
		if (!transduction.equals("")) {
			g.setColor(parent.pref.selectedColor);
			g.fillRect(X1+5,Y1+Height+g.getFontMetrics().getDescent()
			           ,context.getFontMetrics(parent.pref.output).stringWidth(transduction)
			           ,context.getFontMetrics(parent.pref.output).getHeight()+1);
			g.setColor(parent.pref.backgroundColor);
			g.setFont(parent.pref.output);
			g.drawString(transduction,X1+5,Y1+Height+g.getFontMetrics().getHeight());
		}
	}




	public void drawInitial(Graphics g) {
		drawOther(g);
		if (parent.pref.rightToLeft==false)
			g.drawLine(X_in,Y_in,X_in-10,Y_in);
		else g.drawLine(X_out-5,Y_out,X_out+5,Y_out);
	}


	public void drawInitialSelected(Graphics g) {
		drawOtherSelected(g);
		g.setColor(parent.pref.foregroundColor);
		if (parent.pref.rightToLeft==false)
			g.drawLine(X_in-1,Y_in,X_in-10,Y_in);
		else g.drawLine(X_out-5,Y_out,X_out+5,Y_out);
	}

	
	public void draw(Graphics g) {
		g.setFont(parent.pref.input);
		h_ligne=g.getFontMetrics().getHeight();
		descent=g.getFontMetrics().getDescent();
		if (singleDragging==true) {
			// if the box is being dragged just under the mouse,
			// we just draw its frame
			if (TYPE==1) drawFinalSingleDrag(g);
			else if (TYPE==2) drawOtherSingleDrag(g);
			else drawInitialSingleDrag(g);
		}
		else if (selected==true) {
			// if the box was selected before (blue box)
			if (TYPE==1) drawFinalSelected(g);
			else if (TYPE==2) drawOtherSelected(g);
			else drawInitialSelected(g);
		}
		else if (comment==true) {
			// if the box is in comment and not selected
			if (TYPE==1) drawFinal(g);
			else if (TYPE==2) drawOtherComment(g);
			else drawInitial(g);
		} else {
			// the box is normal
			if (TYPE==1) drawFinal(g);
			else if (TYPE==2) drawOther(g);
			else drawInitial(g);
		}
	}








/*
 * The following methods are used to manipulate the box text
 *
 */


	public int test_transduction(char s[],int i)
	{
  	int compteur;

  	if(s[i]!='/') return 0;
  	i--;
  		compteur=0;
  	while(i>=0 && s[i]=='\\')
    	{
      	compteur++;
      	i--;
    	}
  	if((compteur%2)!=0) return 0;
  	return 1;
	}


	public static boolean isAPlusChar(char c) {
		return (c=='+' || c=='\uff0b');
	}
	

	public void tokenizeTextWithoutTransduction(String s) throws BackSlashAtEndOfLineError,MissingGraphNameError,NoClosingQuoteError,NoClosingSupError {
		int L=s.length(),i=0;
		String tmp;
		char ligne[]=new char[10000];
		ligne=s.toCharArray();
		
		while (i<L) {
		  	tmp="";
		  	if (ligne[i]==':') {
		    	// case of a sub graph call
		    	i++;
		    	while ((i<L) && !isAPlusChar(ligne[i])/*(ligne[i]!='+')*/) {
		    		if (ligne[i]=='\\') {
		    	  		tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	  		if (i>=L) {
		    	    		BackSlashAtEndOfLineError e=new BackSlashAtEndOfLineError();
		    	    		throw e;
		    	  		}
		    		}
		    		tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	}
		    	if (tmp.length()==0) {
		    		MissingGraphNameError e=new MissingGraphNameError();
		    		throw e;
		    	}
		    	// if we had a + separator char (even a japanese one), we put a standard + instead
		    	if (i<L) ligne[i]='+';
		    	i++;
		    	n_lignes++;
		    	lines.add(tmp);
		    	greyed.add(new Boolean(true));
		    } else {
		        // all other cases
		    	while ((i<L) && !isAPlusChar(ligne[i])/*(ligne[i]!='+')*/) {
		    		if (ligne[i]=='"') {
		    			// case of a quote expression
		    			tmp=tmp.concat(String.valueOf(ligne[i++]));
		    			while ((i<L) && ligne[i]!='"') {
		    				if (ligne[i]=='\\') {
		    	  				tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	  				if (i>=L) {
		    	    				BackSlashAtEndOfLineError e=new BackSlashAtEndOfLineError();
		    	    				throw e;
		    	  				}
		    				}
		    				tmp=tmp.concat(String.valueOf(ligne[i++]));	
		    			}
		    			if (i>=L) {
		    				NoClosingQuoteError e=new NoClosingQuoteError();
		    	    		throw e;
		    	  		}
		    	  		tmp=tmp.concat(String.valueOf(ligne[i++]));	
		    		}
		    		else if (ligne[i]=='<') {
		    			// case of a <...> expression
		    			tmp=tmp.concat(String.valueOf(ligne[i++]));
		    			while ((i<L) && ligne[i]!='>') {
		    				if (ligne[i]=='\\') {
		    	  				tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	  				if (i>=L) {
		    	    				BackSlashAtEndOfLineError e=new BackSlashAtEndOfLineError();
		    	    				throw e;
		    	  				}
		    				}
		    				tmp=tmp.concat(String.valueOf(ligne[i++]));	
		    			}
		    			if (i>=L) {
		    				NoClosingSupError e=new NoClosingSupError();
		    	    		throw e;
		    	  		}
		    	  		tmp=tmp.concat(String.valueOf(ligne[i++]));	
		    		}
		    		else {
		    			if (ligne[i]=='\\') {
		    	  			tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	  			if (i>=L) {
		    	    			BackSlashAtEndOfLineError e=new BackSlashAtEndOfLineError();
		    	    			throw e;
		    	  			}
		    			}
			    		tmp=tmp.concat(String.valueOf(ligne[i++]));
			    	}
		    	}
		    	if (i<L) ligne[i]='+';
		    	n_lignes++;
		    	lines.add(tmp);
		    	greyed.add(new Boolean(false));
		    	i++;
		    }
		}
	}
	
	
	public int get_h_ligne() {
		context.setFont(parent.pref.input);
		return context.getFontMetrics().getHeight();
	}


	public int get_h_variable_ligne() {
		context.setFont(variableFont);
		return context.getFontMetrics().getHeight();
	}


	public int maxVariableLineWidth() {
		FontMetrics f=context.getFontMetrics(variableFont);
		return f.stringWidth("(");
	}
	
	
	public int maxLineWidth() {
		int i,max=0;
		String s;
		max=0;
		FontMetrics f=context.getFontMetrics(parent.pref.input);
		for (i=0;i<n_lignes;i++) {
			s=(String)lines.get(i);
			if (max<f.stringWidth(s))
	  			max=f.stringWidth(s);
		}
		return max;
	}
	
	
	public void setText(String s) {
		if (TYPE==1) return; // nothing to do if we consider the final state
		contenu=s;
		char ligne[]=new char[10000];
		String tmp="";
		int i,j,L;
		
		ligne=contenu.toCharArray();
		i=0;
		L=contenu.length();
		if (ligne[0]=='$' && (ligne[L-1]=='(' || ligne[L-1]==')')) {
			////////////////////////////////////////////
			// case of $a( or $a)
			////////////////////////////////////////////
			variable=true;
			lines.removeAllElements();
			greyed.removeAllElements();
			lines.add(String.valueOf(ligne[L-1]));
			greyed.add(new Boolean(false));
			transduction=contenu.substring(1,L-1);
			n_lignes=1;
			Height=get_h_variable_ligne()+3;
			Width=maxVariableLineWidth()+5;
		  	Y1=Y-Height/2;
		  	X_out=X+Width+2;
			return;
		}
		variable=false;
		while((i!=L) && (test_transduction(ligne,i) == 0))
      		tmp=tmp.concat(String.valueOf(ligne[i++]));
		transduction="";
  		if (i!=L) {
			j=0;
			i++;
			while (i!=L)
			  transduction=transduction.concat(String.valueOf(ligne[i++]));
		}
		n_lignes=0;
		lines.removeAllElements();
		greyed.removeAllElements();
		try {
			if (!tmp.equals("<E>")) tokenizeTextWithoutTransduction(tmp);
		} catch(BackSlashAtEndOfLineError e) {}
		  catch(MissingGraphNameError e) {}
		  catch(NoClosingQuoteError e) {}
		  catch(NoClosingSupError e) {}
		if (!tmp.equals("<E>")) {
			// dimensions of a full box
			Width=maxLineWidth()+10;
			Height=n_lignes*get_h_ligne()+6;
		}		
		else {
			// dimensions of an empty box
			Height=20;
			Width=15;
		}
	  	Y1=Y-Height/2;
	  	X_out=X+Width+5;
	}


	public void update() {
		// this method is used to resize the box after a font change
		setText(contenu);
	}


	public int strcmp(String a,String b) {
		if (a.compareTo("<E>")==0) {
			if (b.compareTo("<E>")==0)
				return 0;
			else return -1;
		}
		else {
			if (b.compareTo("<E>")==0)
				return 1;
			else return a.compareTo(b);
		}
	}
	
	
	public int partition_pour_quicksort(Vector v,Vector v2,int m, int n) {
		String pivot;
		Object tmp;
		int i = m-1;
		int j = n+1;
		pivot=(String)v.get((m+n)/2);
		while (true) {
  			do j--;
  			while ((j>(m-1))&&(strcmp(pivot,(String)v.get(j)) < 0));
  			do i++;
  			while ((i<n+1)&&(strcmp((String)v.get(i),pivot) < 0));
  			if (i<j) {
    			tmp=v.get(i);
    			v.setElementAt(v.get(j),i);
    			v.setElementAt(tmp,j);
    			tmp=v2.get(i);
    			v2.setElementAt(v2.get(j),i);
    			v2.setElementAt(tmp,j);
  			} else return j;
		}
	}


	public void quicksort(Vector v,Vector v2,int m,int n) {
		int p;
		if (m<n) {
  			p=partition_pour_quicksort(v,v2,m,n);
  			quicksort(v,v2,m,p);
  			quicksort(v,v2,p+1,n);
		}
	}



	public void sortNodeLabel() {
		if (!lines.isEmpty()) {
			quicksort(lines,greyed,0,lines.size()-1);
			contenu=(String)lines.get(0);
			for (int i=1;i<lines.size();i++) {
				contenu=contenu+"+"+(String)lines.get(i);
			}
	//		if (transduction!=null && !transduction.equals(""))
	//			contenu=contenu+"/"+transduction;
		}
	}

} 

/* end of GraphBox */
