 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;


/**
  * Window class used to display a graph
  *
  */
public class GraphFrame extends JInternalFrame implements InternalFrameListener {
  
    	static int openFrameCount = 0;
    	static final int offset = 30;
    	public TextField texte;
	public GraphicalZone graphicalZone;
	public String filename="";
	public String pathname="";
	public boolean modified=false;
	public boolean have_a_name=false;
	JScrollPane scroll;
	boolean empty=false;
	public ComponentListener compListener=null;
	public JToolBar myToolBar;
	
	public UNDOContainer UNDO=new UNDOContainer(this);


    public GraphFrame(boolean e) {
		super("", true, true, true, true);
		empty=e;
		openFrameCount++;
		filename="";
		if (have_a_name) {
			if (modified)
				setTitle(filename+" (Unsaved)");
			else setTitle(filename);
		}
		else {
			if (modified)
				setTitle(" (Unsaved)");
			else setTitle("Graph");
		}
		JPanel mainPanel=new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
		//createToolBar();
		
		JPanel top = new JPanel();
		top.setLayout(new BorderLayout());
		top.add(buildTextPanel(),BorderLayout.NORTH);
		graphicalZone=new GraphicalZone(1188,840,texte,this);
		graphicalZone.setPreferredSize(new Dimension((int)(1188*graphicalZone.scaleFactor),(int)(840*graphicalZone.scaleFactor)));
		scroll=new JScrollPane(graphicalZone);
		scroll.setOpaque(true);
		scroll.getHorizontalScrollBar().setUnitIncrement(20);
		scroll.getVerticalScrollBar().setUnitIncrement(20);
		scroll.setPreferredSize(new Dimension(1188,840));
		top.add(scroll,BorderLayout.CENTER);
		texte.setFont(graphicalZone.pref.input);
		//mainPanel.add(myToolBar,BorderLayout.WEST);
		mainPanel.add(top,BorderLayout.CENTER);
		setContentPane(mainPanel);
		pack();
    		addInternalFrameListener(this);
		setBounds(offset*(openFrameCount%6),offset*(openFrameCount%6),800+40,600+80);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    }



	public void createToolBar() {
		myToolBar=new JToolBar("Tools",SwingConstants.VERTICAL);
		myToolBar.setMargin(new Insets(0,0,0,0));
		JButton save=new JButton(MyCursors.saveIcon);
		
		JButton copy=new JButton(MyCursors.copyIcon);
		JButton cut=new JButton(MyCursors.cutIcon);
		JButton paste=new JButton(MyCursors.pasteIcon);
		
		JToggleButton normal=new JToggleButton(MyCursors.arrowIcon);
		//JToggleButton select=new JToggleButton(MyCursors.selectBoxesIcon);
		//JToggleButton move=new JToggleButton(MyCursors.moveBoxesIcon);
		JToggleButton create=new JToggleButton(MyCursors.createBoxesIcon);
		//JToggleButton edit=new JToggleButton(MyCursors.editBoxesIcon);
		JToggleButton kill=new JToggleButton(MyCursors.killBoxesIcon);
		JToggleButton link=new JToggleButton(MyCursors.linkBoxesIcon);
		JToggleButton reverseLink=new JToggleButton(MyCursors.reverseLinkBoxesIcon);
		JToggleButton openSubgraph=new JToggleButton(MyCursors.openSubgraphIcon);

		JButton configuration=new JButton(MyCursors.configurationIcon);

		save.setToolTipText("Save graph");
		
		copy.setToolTipText("Copy");
		cut.setToolTipText("Cut");
		paste.setToolTipText("Paste");
		
		normal.setToolTipText("Normal editing mode");
		//select.setToolTipText("Select boxes");
		//move.setToolTipText("Move boxes");
		create.setToolTipText("Create a new box");
		//edit.setToolTipText("Edit a box");
		kill.setToolTipText("Remove a box");
		link.setToolTipText("Link boxes");
		reverseLink.setToolTipText("Reversed link between boxes");
		openSubgraph.setToolTipText("Open a sub-graph");

		configuration.setToolTipText("Graph configuration");
		
		save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				UnitexFrame.mainFrame.saveGraph(UnitexFrame.currentFrame);
			}
		});
		copy.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final ActionEvent E=e;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						graphicalZone.texte.specialCopy.actionPerformed(E);
						repaint();
					}
				});
			}
		});
		cut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final ActionEvent E=e;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						graphicalZone.texte.cut.actionPerformed(E);
						repaint();
					}
				});
			}
		});
		paste.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final ActionEvent E=e;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						graphicalZone.texte.specialPaste.actionPerformed(E);
						repaint();
					}
				});
			}
		});
		normal.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphicalZone.setCursor(MyCursors.normalCursor);
				graphicalZone.EDITING_MODE=MyCursors.NORMAL;
				repaint();
			}
		});
		/*select.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphicalZone.setCursor(MyCursors.selectBoxesCursor);
				graphicalZone.EDITING_MODE=MyCursors.SELECT_BOXES;
			}
		});*/
		/*move.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphicalZone.setCursor(MyCursors.moveBoxesCursor);
				graphicalZone.EDITING_MODE=MyCursors.MOVE_BOXES;
			}
		});*/
		create.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphicalZone.setCursor(MyCursors.createBoxesCursor);
				graphicalZone.EDITING_MODE=MyCursors.CREATE_BOXES;
				repaint();
			}
		});
		/*edit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphicalZone.setCursor(MyCursors.editBoxesCursor);
				graphicalZone.EDITING_MODE=MyCursors.EDIT_BOXES;
			}
		});*/
		kill.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphicalZone.setCursor(MyCursors.killBoxesCursor);
				graphicalZone.EDITING_MODE=MyCursors.KILL_BOXES;
	    			graphicalZone.unSelectAllBoxes();
				texte.validateTextField();
			}
		});
		link.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphicalZone.setCursor(MyCursors.linkBoxesCursor);
				graphicalZone.EDITING_MODE=MyCursors.LINK_BOXES;
				repaint();
			}
		});
		reverseLink.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphicalZone.setCursor(MyCursors.reverseLinkBoxesCursor);
				graphicalZone.EDITING_MODE=MyCursors.REVERSE_LINK_BOXES;
				repaint();
			}
		});
		openSubgraph.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphicalZone.setCursor(MyCursors.openSubgraphCursor);
				graphicalZone.EDITING_MODE=MyCursors.OPEN_SUBGRAPH;
				texte.validateTextField();
			}
		});
		configuration.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						new GraphPresentationMenu();
					}
				});
		}});
			
		ButtonGroup bg=new ButtonGroup();
		bg.add(normal);
		//bg.add(select);
		//bg.add(move);
		bg.add(create);
		//bg.add(edit);
		bg.add(kill);
		bg.add(link);
		bg.add(reverseLink);
		bg.add(openSubgraph);
		
		
		normal.setSelected(true);
		myToolBar.add(save);
		
		myToolBar.addSeparator();
		myToolBar.addSeparator();

		myToolBar.add(copy);
		myToolBar.add(cut);
		myToolBar.add(paste);

		myToolBar.addSeparator();
		myToolBar.addSeparator();

		myToolBar.add(normal);
		//myToolBar.add(select);
		//myToolBar.add(move);
		myToolBar.add(create);
		//myToolBar.add(edit);
		myToolBar.add(kill);
		myToolBar.add(link);
		myToolBar.add(reverseLink);
		myToolBar.add(openSubgraph);

		myToolBar.addSeparator();
		myToolBar.addSeparator();

		myToolBar.add(configuration);
	}



    private JPanel buildTextPanel() {
        JPanel p = new JPanel();
        p.setLayout( new LabeledPairLayout() );
		JLabel toLabel = new JLabel("",JLabel.RIGHT);
		texte = new TextField(25,this);
		p.add(toLabel, "label");
		p.add(texte, "field");
		return p;
    }




class LabeledPairLayout implements LayoutManager {

	Vector labels = new Vector();
	Vector fields = new Vector();
	int yGap = 2;
	int xGap = 2;


	public void addLayoutComponent(String s, Component c) {
		if (s.equals("label")) {
			labels.addElement(c);
		}
		else {
			fields.addElement(c);
		}
	}


	public void layoutContainer(Container c) {
		Insets insets = c.getInsets();
		int labelWidth = 0;
		Enumeration labelIter = labels.elements();
		while(labelIter.hasMoreElements()) {
			JComponent comp = (JComponent)labelIter.nextElement();
			labelWidth = Math.max( labelWidth, comp.getPreferredSize().width );
		}
		int yPos = insets.top;
		
		Enumeration fieldIter = fields.elements();
		labelIter = labels.elements();
		while(labelIter.hasMoreElements() && fieldIter.hasMoreElements()) {
			JComponent label = (JComponent)labelIter.nextElement();
			JComponent field = (JComponent)fieldIter.nextElement();
			int height = Math.max(label.getPreferredSize().height, field.getPreferredSize().height);
			label.setBounds( insets.left, yPos, labelWidth, height );
			field.setBounds( insets.left + labelWidth + xGap, 
				 yPos, 
				 c.getSize().width - (labelWidth +xGap + insets.left + insets.right), 
				 height );
			yPos += (height + yGap);
		}
	}


	public Dimension minimumLayoutSize(Container c) {
		Insets insets = c.getInsets();
		int labelWidth = 0;
		Enumeration labelIter = labels.elements();
		while(labelIter.hasMoreElements()) {
			JComponent comp = (JComponent)labelIter.nextElement();
			labelWidth = Math.max( labelWidth, comp.getPreferredSize().width );
		}
		int yPos = insets.top;
		labelIter = labels.elements();
		Enumeration fieldIter = fields.elements();
		while(labelIter.hasMoreElements() && fieldIter.hasMoreElements()) {
			JComponent label = (JComponent)labelIter.nextElement();
			JComponent field = (JComponent)fieldIter.nextElement();
			int height = Math.max(label.getPreferredSize().height, field.getPreferredSize().height);
	      yPos += (height + yGap);
	    }
	    return new Dimension( labelWidth * 3 , yPos );
	}


	public Dimension preferredLayoutSize(Container c) {
		Dimension d = minimumLayoutSize(c);
		d.width *= 2;
		return d;
	}


	public void removeLayoutComponent(Component c) {}

}



	public void internalFrameActivated(InternalFrameEvent e) {
		UnitexFrame.currentFrame=this;
		texte.requestFocus();
		texte.getCaret().setVisible(true);
	}
	public void internalFrameClosed(InternalFrameEvent e) {
		UnitexFrame.frames.remove(this);
		if (UnitexFrame.frames.isEmpty())
			UnitexFrame.currentFrame=null;
	}
	public void internalFrameClosing(InternalFrameEvent e) {
		if (modified) {
			Object[] options = {"Save","Don't save","Cancel"};
			int n=JOptionPane.showOptionDialog(this,"Graph has been modified. Do you want to save it ?"
			,""
			,JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE,null,options,options[0]);
			if (n==JOptionPane.CLOSED_OPTION) return;
			if (n==0) {
 				if (!UnitexFrame.mainFrame.saveGraph(this)) return;
 			}
 			if (n!=2) {
 				this.setVisible(false);
 				this.dispose();
 				return;
 			}
 			else {
 				this.setVisible(true);
 				try { 
				    this.setSelected(true); 
				    this.setIcon(false);
				} catch (java.beans.PropertyVetoException e2) {}
 				return;
 			}
		}
		this.setVisible(false);
		this.dispose();
	}
	public void internalFrameDeactivated(InternalFrameEvent e) {
	}
	public void internalFrameDeiconified(InternalFrameEvent e) {
	}
	public void internalFrameIconified(InternalFrameEvent e) {
	}
	public void internalFrameOpened(InternalFrameEvent e) {
	}


	public void reSize(int x,int y) {
		graphicalZone.Width=x;
		graphicalZone.Height=y;
		graphicalZone.setPreferredSize(new Dimension(x,y));
		graphicalZone.revalidate();
		graphicalZone.repaint();
	}
	
	
	public void setModified(boolean b) {
		modified=b;
		if (have_a_name) {
			if (modified)
				setTitle(filename+" (Unsaved)");
			else setTitle(filename);
		}
		else {
			if (modified)
				setTitle(" (Unsaved)");
			else setTitle("Graph");
		}
	}
	
	
	public void setScaleFactor(double d) {
		graphicalZone.scaleFactor=d;
		graphicalZone.setPreferredSize(new Dimension((int)(graphicalZone.Width*graphicalZone.scaleFactor),
		                                   (int)(graphicalZone.Height*graphicalZone.scaleFactor)));
		graphicalZone.revalidate();
		graphicalZone.repaint();
	}


	public void sortNodeLabel() {
		if (graphicalZone.selectedBoxes.isEmpty()) return;
		for (int i=0;i<graphicalZone.selectedBoxes.size();i++) {
			GraphBox g=(GraphBox)graphicalZone.selectedBoxes.get(i);
			g.sortNodeLabel();
		}
		graphicalZone.unSelectAllBoxes();
		texte.initText("");
		graphicalZone.repaint();
	}

	
	public void changeAntialiasingValue() {
		graphicalZone.pref.antialiasing=!graphicalZone.pref.antialiasing;
		graphicalZone.repaint();
	}
	

} /* end of GraphFrame */





















	class BackSlashAtEndOfLineError extends Exception {
		public BackSlashAtEndOfLineError() {
			super(); 
		}
	}


	class MissingGraphNameError extends Exception {
		public MissingGraphNameError() {
			super(); 
		}
	}


	class NoClosingQuoteError extends Exception {
		public NoClosingQuoteError() {
			super(); 
		}
	}


	class NoClosingSupError extends Exception {
		public NoClosingSupError() {
			super(); 
		}
	}


