 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.* ;

/*
 * This class provides methods for loading and saving graphs
 *
 */
public class GraphIO {
	
	Vector boxes;
	Preferences pref=new Preferences();
	int width,height;
	int n_boxes;
	
	
	// This method loads the graph "name" and returns a vector containing the GraphBox objects created
	public static GraphIO loadGraph(String name) {
		GraphIO res=new GraphIO();
		File infile;
		FileInputStream source;
		infile=new File(name);
		if (!infile.exists()) {
			JOptionPane.showMessageDialog(null,"Cannot find "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return null;
		}
		if (!infile.canRead()) {
			JOptionPane.showMessageDialog(null,"Cannot read "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return null;
		}
		if (infile.length()<=2) {
			JOptionPane.showMessageDialog(null,name+" is empty","Error",JOptionPane.ERROR_MESSAGE); 
			return null;
		}
		try {
			source=new FileInputStream(infile);
			char c=read_char(source);
			if (c!=0xFEFF) {
				JOptionPane.showMessageDialog(null,name+" is not a Unicode Little-Endian graph","Error",JOptionPane.ERROR_MESSAGE);
				return null;
			}
			res.skipLine(source); // ignoring #...
			res.readSize(source);
			res.readInputFont(source);
			res.readOutputFont(source);
			res.readBackgroundColor(source);
			res.readForegroundColor(source);
			res.readSubgraphColor(source);
			res.readCommentColor(source);
			res.readSelectedColor(source);
			res.skipLine(source); // ignoring DBOXES
			res.readDrawFrame(source);
			res.readDate(source);
			res.readFile(source);
			res.readDirectory(source);
			res.readRightToLeft(source);
			res.skipLine(source); // ignoring DRST
			res.skipLine(source); // ignoring FITS
			res.skipLine(source); // ignoring PORIENT
			res.skipLine(source); // ignoring #
			res.readBoxNumber(source);
			res.boxes=new Vector();
			// adding initial state
			res.boxes.add(new GraphBox(0,0,0,null,null));
			// adding final state
			res.boxes.add(new GraphBox(0,0,1,null,null));
			// adding other states
			for (int i=2;i<res.n_boxes;i++)
				res.boxes.add(new GraphBox(0,0,2,null,null));
			for (int i=0;i<res.n_boxes;i++)
				res.readLine(source,i);
			source.close();
		} catch (FileNotFoundException e) {e.printStackTrace(); return null;}
		  catch (IOException e) {e.printStackTrace(); return null;}
		return res;
	}
	
	
	public void skipLine(FileInputStream f) {
		char c;
		String s="";
		while ((c=read_char(f))!='\n') {
			s=s+c;
		}
	}
	
	
	public static int toInt(byte b) {
		if (b>0) return b;
		else return 256+b;
	}
	
	
	public static char read_char(FileInputStream f) {
		byte[] b=new byte[2];
		char c;
		try {
			f.read(b);
		} catch(IOException e) {e.printStackTrace();}
		c=(char)(toInt(b[1])*256+toInt(b[0]));
		if (c==0x0d) {
			read_char(f);
			c='\n';
		}
		return c;
	}


	public static void read_char(FileInputStream f,int n) {
		for (int i=0;i<n;i++)
			read_char(f);
	}
	
	
	public static boolean isDigit(char c) {
		return (c>='0' && c<='9');
	}
	
	
	public void readSize(FileInputStream f) {
		// skipping the chars preceeding the width and height
		read_char(f,5);
		char c;
		// reading width
		width=0;
		while (isDigit((c=read_char(f))))
			width=width*10+(c-'0');
		// reading height
		while (isDigit((c=read_char(f))))
			height=height*10+(c-'0');
	}


	public void readInputFont(FileInputStream f) {
		read_char(f,5);
		String s="";
		char c;
		while ((c=read_char(f))!=':')
			s=s+c;
		boolean bold=(read_char(f)=='B');
		boolean italic=(read_char(f)=='I');
		int size=0;
		while (isDigit((c=read_char(f))))
			size=size*10+(c-'0');
		pref.inputSize=size;
		int style;
		if (bold && italic) style=Font.BOLD|Font.ITALIC;
		else if (bold) style=Font.BOLD;
		else if (italic) style=Font.ITALIC;
		else style=Font.PLAIN;
		pref.input=new Font(s,style,(int)(size/0.72));
	}


	public void readOutputFont(FileInputStream f) {
		read_char(f,6);
		String s="";
		char c;
		while ((c=read_char(f))!=':')
			s=s+c;
		boolean bold=(read_char(f)=='B');
		boolean italic=(read_char(f)=='I');
		int size=0;
		while (isDigit((c=read_char(f))))
			size=size*10+(c-'0');
		pref.outputSize=size;
		int style;
		if (bold && italic) style=Font.BOLD|Font.ITALIC;
		else if (bold) style=Font.BOLD;
		else if (italic) style=Font.ITALIC;
		else style=Font.PLAIN;
		pref.output=new Font(s,style,(int)(size/0.72));
	}
	
	
	public void readBackgroundColor(FileInputStream f) {
		read_char(f,7);
		char c;
		int n=0;
		while (isDigit((c=read_char(f))))
			n=n*10+(c-'0');
		pref.backgroundColor=new Color(n);
	}


	public void readForegroundColor(FileInputStream f) {
		read_char(f,7);
		char c;
		int n=0;
		while (isDigit((c=read_char(f))))
			n=n*10+(c-'0');
		pref.foregroundColor=new Color(n);
	}


	public void readSubgraphColor(FileInputStream f) {
		read_char(f,7);
		char c;
		int n=0;
		while (isDigit((c=read_char(f))))
			n=n*10+(c-'0');
		pref.subgraphColor=new Color(n);
	}


	public void readSelectedColor(FileInputStream f) {
		read_char(f,7);
		char c;
		int n=0;
		while (isDigit((c=read_char(f))))
			n=n*10+(c-'0');
		pref.selectedColor=new Color(n);
	}


	public void readCommentColor(FileInputStream f) {
		read_char(f,7);
		char c;
		int n=0;
		while (isDigit((c=read_char(f))))
			n=n*10+(c-'0');
		pref.commentColor=new Color(n);
	}


	public void readDrawFrame(FileInputStream f) {
		read_char(f,7);
		pref.frame=(read_char(f)=='y');
		read_char(f);
	}


	public void readDate(FileInputStream f) {
		read_char(f,6);
		pref.date=(read_char(f)=='y');
		read_char(f);
	}


	public void readFile(FileInputStream f) {
		read_char(f,6);
		pref.filename=(read_char(f)=='y');
		read_char(f);
	}


	public void readDirectory(FileInputStream f) {
		read_char(f,5);
		pref.pathname=(read_char(f)=='y');
		read_char(f);
	}


	public void readRightToLeft(FileInputStream f) {
		read_char(f,5);
		pref.rightToLeft=(read_char(f)=='y');
		read_char(f);
	}


	public void readBoxNumber(FileInputStream f) {
		char c;
		n_boxes=0;
		while (isDigit((c=read_char(f))))
			n_boxes=n_boxes*10+(c-'0');
	}


	public void readLine(FileInputStream f,int n) {
		GraphBox g=(GraphBox)boxes.get(n);
		if (read_char(f)=='s') {
			// is a "s" was read, then we read the " char
			read_char(f);
		}
		String s="";
		char c;
		while ((c=read_char(f))!='"') {
			if (c=='\\') {
				c=read_char(f);
				if (c!='\\') {
					// case of \: \+ and \"
					if (c=='"') s=s+c;
					else s=s+"\\"+c;
				} 
				else {
					// case of \\\" that must must be transformed into \"
					c=read_char(f);
					if (c=='\\') {
						// we are in the case \\\" -> \"
						c=read_char(f);
						s=s+"\\"+c;
					}
					else {
						// we are in the case \\a -> \\a
						s=s+"\\\\"+c;
					}
				}
			}
			else s=s+c;
		}
		// skipping the space after "
		read_char(f);
		// reading the X coordinate
		int x=0;
		while (isDigit((c=read_char(f))))
			x=x*10+(c-'0');
		// reading the Y coordinate
		int y=0;
		while (isDigit((c=read_char(f))))
			y=y*10+(c-'0');
		g.X=x;
		g.Y=y;
		g.X1=g.X; 
		g.Y1=g.Y;
		g.X_in=g.X;
	    	g.Y_in=g.Y;
	    	g.X_out=g.X+g.Width+5;
	    	g.Y_out=g.Y_in;
		if (n!=1) {
			// 1 is the final state, which content is <E>
			g.contenu=s; // we will need to call g.update() to size the box according to the text
		}
		else {
			g.contenu="<E>";
			g.X_in=g.X; g.Y_in=g.Y;
			g.X1=g.X; g.Y1=g.Y-10;
			g.Y_out=g.Y_in;
			g.X_out=g.X_in+25;
		}
		int trans=0;
		while (isDigit((c=read_char(f))))
			trans=trans*10+(c-'0');
		for (int j=0;j<trans;j++) {
			int dest=0;
			while (isDigit((c=read_char(f))))
				dest=dest*10+(c-'0');
			g.addTransitionTo((GraphBox)boxes.get(dest));
		}
		// skipping the end-of-line
		read_char(f);
	}


	public void saveGraph(String name) {
		File outfile;
		FileOutputStream dest;
		outfile=new File(name);
		try {
			if (!outfile.exists())
				outfile.createNewFile();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null,"Cannot write "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		if (!outfile.canWrite()) {
			JOptionPane.showMessageDialog(null,"Cannot write "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		try {
			dest=new FileOutputStream(outfile);
			writeOneChar(dest,(char)0xFEFF);
			write_string(dest,"#Unigraph\n");
			write_string(dest,"SIZE "+String.valueOf(width)+" "+String.valueOf(height)+"\n");
			write_string(dest,"FONT "+pref.input.getName()+":");
			if (pref.input.getStyle()==Font.PLAIN)
				write_string(dest,"  ");
			else if (pref.input.getStyle()==Font.BOLD)
				write_string(dest,"B ");
			else if (pref.input.getStyle()==Font.ITALIC)
				write_string(dest," I");
			else write_string(dest,"BI");
			write_string(dest,String.valueOf(pref.inputSize)+"\n");
			write_string(dest,"OFONT "+pref.output.getName()+":");
			if (pref.output.getStyle()==Font.PLAIN)
				write_string(dest,"  ");
			else if (pref.output.getStyle()==Font.BOLD)
				write_string(dest,"B ");
			else if (pref.output.getStyle()==Font.ITALIC)
				write_string(dest," I");
			else write_string(dest,"BI");
			write_string(dest,String.valueOf(pref.outputSize)+"\n");
			write_string(dest,"BCOLOR "+String.valueOf(16777216+pref.backgroundColor.getRGB())+"\n");
			write_string(dest,"FCOLOR "+String.valueOf(16777216+pref.foregroundColor.getRGB())+"\n");
			write_string(dest,"ACOLOR "+String.valueOf(16777216+pref.subgraphColor.getRGB())+"\n");
			write_string(dest,"SCOLOR "+String.valueOf(16777216+pref.commentColor.getRGB())+"\n");
			write_string(dest,"CCOLOR "+String.valueOf(16777216+pref.selectedColor.getRGB())+"\n");
			write_string(dest,"DBOXES y\n");
			if (pref.frame)	write_string(dest,"DFRAME y\n");
			else write_string(dest,"DFRAME n\n");
			if (pref.date)	write_string(dest,"DDATE y\n");
			else write_string(dest,"DDATE n\n");
			if (pref.filename) write_string(dest,"DFILE y\n");
			else write_string(dest,"DFILE n\n");
			if (pref.pathname) write_string(dest,"DDIR y\n");
			else write_string(dest,"DDIR n\n");
			if (pref.rightToLeft) write_string(dest,"DRIG y\n");
			else write_string(dest,"DRIG n\n");
			write_string(dest,"DRST n\n");
			write_string(dest,"FITS 100\n");
			write_string(dest,"PORIENT L\n");
			write_string(dest,"#\n");
			n_boxes=boxes.size();
			write_string(dest,String.valueOf(n_boxes)+"\n");
			for (int i=0;i<n_boxes;i++) {
				GraphBox g=(GraphBox)boxes.get(i);
				writeOneChar(dest,'"');
				if (g.TYPE!=1) write_content(dest,g.contenu);
				int N=g.transitions.size();
				write_string(dest,"\" "+String.valueOf(g.X)+" "+String.valueOf(g.Y)+" "+String.valueOf(N)+" ");
				for (int j=0;j<N;j++) {
					GraphBox tmp=(GraphBox)g.transitions.get(j);
					write_string(dest,String.valueOf(boxes.indexOf(tmp))+" ");
				}
				writeOneChar(dest,'\n');
			}
			dest.close();
		} catch (FileNotFoundException e) {e.printStackTrace();}
		  catch (IOException e) {e.printStackTrace();}
	}
	
	
	public void writeOneChar(FileOutputStream f,char c) {
		byte[] b=new byte[2];
		b[1]=(byte)(c/256);
		b[0]=(byte)(c%256);
		if (c=='\n') {
			writeOneChar(f,(char)0x0d);
		}
		try {
			f.write(b[0]);
			f.write(b[1]);
		} catch(IOException e) {e.printStackTrace();}
		
	}


	public void write_string(FileOutputStream f,String s) {
		int L=s.length();
		for (int i=0;i<L;i++)
			writeOneChar(f,s.charAt(i));
	}


	public void write_content(FileOutputStream f,String s) {
		int L=s.length();
		char c;
		for (int i=0;i<L;i++) {
			c=s.charAt(i);
			if (c=='"') {
				// case of char "
				if (i==0 || s.charAt(i-1)!='\\') {
					// the " is the "abc" one; it must be saved as \"
					writeOneChar(f,'\\');
					writeOneChar(f,'"');
				}
				else {
					// it is the \" char that must be saved as \\\"
					// we only write 2 \ because the third has been saved at the pos i-1
					writeOneChar(f,'\\');
					writeOneChar(f,'\\');
					writeOneChar(f,'"');
				}
			}
			else {
				writeOneChar(f,c);
			}
		}
	}


	// This method loads the sentence graph "name" and returns a vector containing the FstGraphBox objects created
	public static GraphIO loadSentenceGraph(String name) {
		GraphIO res=new GraphIO();
		File infile;
		FileInputStream source;
		infile=new File(name);
		if (!infile.exists()) {
			JOptionPane.showMessageDialog(null,"Cannot find "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return null;
		}
		if (!infile.canRead()) {
			JOptionPane.showMessageDialog(null,"Cannot read "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return null;
		}
		if (infile.length()<=2) {
			JOptionPane.showMessageDialog(null,name+" is empty","Error",JOptionPane.ERROR_MESSAGE); 
			return null;
		}
		try {
			source=new FileInputStream(infile);
			char c=read_char(source);
			if (c!=0xFEFF) {
				JOptionPane.showMessageDialog(null,name+" is not a Unicode Little-Endian graph","Error",JOptionPane.ERROR_MESSAGE);
				return null;
			}
			res.skipLine(source); // ignoring #...
			res.readSize(source);
			res.readInputFont(source);
			res.readOutputFont(source);
			res.readBackgroundColor(source);
			res.readForegroundColor(source);
			res.readSubgraphColor(source);
			res.readCommentColor(source);
			res.readSelectedColor(source);
			res.skipLine(source); // ignoring DBOXES
			res.readDrawFrame(source);
			res.readDate(source);
			res.readFile(source);
			res.readDirectory(source);
			res.readRightToLeft(source);
			res.skipLine(source); // ignoring DRST
			res.skipLine(source); // ignoring FITS
			res.skipLine(source); // ignoring PORIENT
			res.skipLine(source); // ignoring #
			res.readBoxNumber(source);
			res.boxes=new Vector();
			// adding initial state
			res.boxes.add(new FstGraphBox(0,0,0,null,null));
			// adding final state
			res.boxes.add(new FstGraphBox(0,0,1,null,null));
			// adding other states
			for (int i=2;i<res.n_boxes;i++)
				res.boxes.add(new FstGraphBox(0,0,2,null,null));
			for (int i=0;i<res.n_boxes;i++)
				res.readSentenceGraphLine(source,i);
			source.close();
		} catch (FileNotFoundException e) {e.printStackTrace(); return null;}
		  catch (IOException e) {e.printStackTrace(); return null;}
		return res;
	}


	public void readSentenceGraphLine(FileInputStream f,int n) {
		FstGraphBox g=(FstGraphBox)boxes.get(n);
		if (read_char(f)=='s') {
			// is a "s" was read, then we read the " char
			read_char(f);
		}
		String s="";
		char c;
		while ((c=read_char(f))!='"') {
			if (c=='\\') {
				c=read_char(f);
				if (c!='\\') {
					// case of \: \+ and \"
					if (c=='"') s=s+c;
					else s=s+"\\"+c;
				} 
				else {
					// case of \\\" that must must be transformed into \"
					c=read_char(f);
					if (c=='\\') {
						// we are in the case \\\" -> \"
						c=read_char(f);
						s=s+"\\"+c;
					}
					else {
						// we are in the case \\a -> \\a
						s=s+"\\\\"+c;
					}
				}
			}
			else s=s+c;
		}
		// skipping the space after "
		read_char(f);
		// reading the X coordinate
		int x=0;
		while (isDigit((c=read_char(f))))
			x=x*10+(c-'0');
		// reading the Y coordinate
		int y=0;
		while (isDigit((c=read_char(f))))
			y=y*10+(c-'0');
		g.X=x;
		g.Y=y;
		g.X1=g.X; 
		g.Y1=g.Y;
		g.X_in=g.X;
	    	g.Y_in=g.Y;
	    	g.X_out=g.X+g.Width+5;
	    	g.Y_out=g.Y_in;

		if (n!=1) {
			// 1 is the final state, which content is <E>
			g.contenu=s; // we will need to call g.update() to size the box according to the text
		} 
		else {
			g.contenu="<E>";
			g.X_in=g.X; g.Y_in=g.Y;
			g.X1=g.X; g.Y1=g.Y-10;
			g.Y_out=g.Y_in;
			g.X_out=g.X_in+25;
		}
		int trans=0;
		while (isDigit((c=read_char(f))))
			trans=trans*10+(c-'0');
		for (int j=0;j<trans;j++) {
			int dest=0;
			while (isDigit((c=read_char(f))))
				dest=dest*10+(c-'0');
			g.addTransitionTo((FstGraphBox)boxes.get(dest));
		}
		// skipping the end-of-line
		read_char(f);
	}



	public void saveSentenceGraph(String name) {
		File outfile;
		FileOutputStream dest;
		outfile=new File(name);
		try {
			if (!outfile.exists())
				outfile.createNewFile();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null,"Cannot write "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		if (!outfile.canWrite()) {
			JOptionPane.showMessageDialog(null,"Cannot write "+name,"Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		try {
			dest=new FileOutputStream(outfile);
			writeOneChar(dest,(char)0xFEFF);
			write_string(dest,"#Unigraph\n");
			write_string(dest,"SIZE "+String.valueOf(width)+" "+String.valueOf(height)+"\n");
			write_string(dest,"FONT "+pref.input.getName()+":");
			if (pref.input.getStyle()==Font.PLAIN)
				write_string(dest,"  ");
			else if (pref.input.getStyle()==Font.BOLD)
				write_string(dest,"B ");
			else if (pref.input.getStyle()==Font.ITALIC)
				write_string(dest," I");
			else write_string(dest,"BI");
			write_string(dest,String.valueOf(pref.inputSize)+"\n");
			write_string(dest,"OFONT "+pref.output.getName()+":");
			if (pref.output.getStyle()==Font.PLAIN)
				write_string(dest,"  ");
			else if (pref.output.getStyle()==Font.BOLD)
				write_string(dest,"B ");
			else if (pref.output.getStyle()==Font.ITALIC)
				write_string(dest," I");
			else write_string(dest,"BI");
			write_string(dest,String.valueOf(pref.outputSize)+"\n");
			write_string(dest,"BCOLOR "+String.valueOf(16777216+pref.backgroundColor.getRGB())+"\n");
			write_string(dest,"FCOLOR "+String.valueOf(16777216+pref.foregroundColor.getRGB())+"\n");
			write_string(dest,"ACOLOR "+String.valueOf(16777216+pref.subgraphColor.getRGB())+"\n");
			write_string(dest,"SCOLOR "+String.valueOf(16777216+pref.commentColor.getRGB())+"\n");
			write_string(dest,"CCOLOR "+String.valueOf(16777216+pref.selectedColor.getRGB())+"\n");
			write_string(dest,"DBOXES y\n");
			if (pref.frame)	write_string(dest,"DFRAME y\n");
			else write_string(dest,"DFRAME n\n");
			if (pref.date)	write_string(dest,"DDATE y\n");
			else write_string(dest,"DDATE n\n");
			if (pref.filename) write_string(dest,"DFILE y\n");
			else write_string(dest,"DFILE n\n");
			if (pref.pathname) write_string(dest,"DDIR y\n");
			else write_string(dest,"DDIR n\n");
			if (pref.rightToLeft) write_string(dest,"DRIG y\n");
			else write_string(dest,"DRIG n\n");
			write_string(dest,"DRST n\n");
			write_string(dest,"FITS 100\n");
			write_string(dest,"PORIENT L\n");
			write_string(dest,"#\n");
			n_boxes=boxes.size();
			write_string(dest,String.valueOf(n_boxes)+"\n");
			for (int i=0;i<n_boxes;i++) {
				FstGraphBox g=(FstGraphBox)boxes.get(i);
				writeOneChar(dest,'"');
				if (g.TYPE!=1) write_content(dest,g.contenu);
				int N=g.transitions.size();
				write_string(dest,"\" "+String.valueOf(g.X)+" "+String.valueOf(g.Y)+" "+String.valueOf(N)+" ");
				for (int j=0;j<N;j++) {
					FstGraphBox tmp=(FstGraphBox)g.transitions.get(j);
					write_string(dest,String.valueOf(boxes.indexOf(tmp))+" ");
				}
				writeOneChar(dest,'\n');
			}
			dest.close();
		} catch (FileNotFoundException e) {e.printStackTrace();}
		  catch (IOException e) {e.printStackTrace();}
	}


}