 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;

/*
 * This class describes the graph preference menu
 *
 */
public class GraphPreferenceMenu extends JDialog {

	JPanel panel=new JPanel();
	JPanel centerPanel=new JPanel();
	JPanel southPanel=new JPanel();
	JPanel buttonPanel=new JPanel();
	JPanel horizontal=new JPanel();
	JPanel vertical=new JPanel();
	JButton top=new JButton("Top");
	JButton centerH=new JButton("Center");
	JButton bottom=new JButton("Bottom");
	JButton left=new JButton("Left");
	JButton centerV=new JButton("Center");
	JButton right=new JButton("Right");
	JButton OK=new JButton("OK");
	JButton CANCEL=new JButton("Cancel");
	GridBagLayout horizGridBag=new GridBagLayout();
	GridBagLayout vertGridBag=new GridBagLayout();
	GridBagConstraints horizConstraints = new GridBagConstraints();
	GridBagConstraints vertConstraints = new GridBagConstraints();
	public JCheckBox checkBox=new JCheckBox();
	public NumericTextField nPixels=new NumericTextField(3,String.valueOf(nPix));
	
	public static GraphPreferenceMenu pref;
	
	public static int nPix=30;
	
	public GraphPreferenceMenu(boolean isGrid,int nPix) {
		super(UnitexFrame.mainFrame,"Alignment",true);
		constructPanel();
		setContentPane(panel);
		pack();
		setResizable(false);
		pref=this;
		nPixels.setEditable(isGrid);
		nPixels.setText(new Integer(nPix).toString());
		checkBox.setSelected(isGrid);
		showPreferences();
	}


	public void showPreferences() {
		setLocationRelativeTo(UnitexFrame.mainFrame);
		this.setVisible(true);
	}
	
	void constructPanel() {
		panel.setBorder(new EmptyBorder(10, 10, 10, 10));
		panel.setLayout(new BorderLayout());
		
		top.addMouseListener(new HTopListener());
		centerH.addMouseListener(new HCenterListener());
		bottom.addMouseListener(new HBottomListener());
		left.addMouseListener(new VLeftListener());
		centerV.addMouseListener(new VCenterListener());
		right.addMouseListener(new VRightListener());
		OK.addMouseListener(new OKListener());
		CANCEL.addMouseListener(new CANCELListener());

		horizontal.setBorder(new TitledBorder("Horizontal"));
		horizontal.setLayout(horizGridBag);
		horizConstraints.fill=GridBagConstraints.NONE;
		horizConstraints.gridwidth=GridBagConstraints.REMAINDER;
		horizGridBag.setConstraints(top,horizConstraints);
		horizontal.add(top);
		horizConstraints.gridwidth=GridBagConstraints.REMAINDER;
		horizGridBag.setConstraints(centerH,horizConstraints);
		horizontal.add(centerH);
		horizConstraints.gridwidth=GridBagConstraints.REMAINDER;
		horizGridBag.setConstraints(bottom,horizConstraints);
		horizontal.add(bottom);

		vertical.setBorder(new TitledBorder("Vertical"));
		vertical.setLayout(new GridLayout(3,1));
		JPanel leftPanel=new JPanel();
		leftPanel.setLayout(new GridLayout(1,2));
		leftPanel.add(new JPanel());
		leftPanel.add(left);
		vertical.add(leftPanel);
		JPanel centerPane=new JPanel();
		centerPane.setLayout(vertGridBag);
		vertConstraints.fill=GridBagConstraints.NONE;
		vertGridBag.setConstraints(centerV,vertConstraints);
		centerPane.add(centerV);
		vertical.add(centerPane);
		JPanel rightPanel=new JPanel();
		rightPanel.setLayout(new GridLayout(1,2));
		rightPanel.add(right);
		rightPanel.add(new JPanel());
		vertical.add(rightPanel);
		centerPanel.add(horizontal);
		centerPanel.add(vertical);
		
		southPanel.setLayout(new FlowLayout());
		checkBox.addItemListener(new CheckBoxListener());
		southPanel.add(checkBox);
		southPanel.add(new JLabel("Use Grid, every"));
		southPanel.add(nPixels);
		southPanel.add(new JLabel("pixels"));
		
		buttonPanel.setLayout(new FlowLayout());
		buttonPanel.add(OK);
		buttonPanel.add(CANCEL);
		
		panel.add(centerPanel,BorderLayout.NORTH);
		panel.add(southPanel,BorderLayout.CENTER);
		panel.add(buttonPanel,BorderLayout.SOUTH);
	}
	
}


class NumericTextField extends JTextField {
	
	NumericTextField(int c,String s) {
		super(c);
		setEditable(true);
		setText(s);
		setEditable(false);
	}
	
	public Document createDefaultModel() {
		return new NumericTextDocument();
	}

}


class NumericTextDocument extends PlainDocument {

	public void insertString(int offs,String s,AttributeSet a) throws BadLocationException {
		int i;
		if (s==null) return;
		char c[]=s.toCharArray();
		for (i=0;i<c.length;i++)
			if (c[i]<'0' || c[i]>'9') return;
		super.insertString(offs,s,a);
	}
}


class HTopListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		GraphPreferenceMenu.pref.setVisible(false);
		if (UnitexFrame.currentFrame==null) return;
		UnitexFrame.currentFrame.graphicalZone.HTopAlign();
		UnitexFrame.currentFrame.setModified(true);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
}


class HCenterListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		GraphPreferenceMenu.pref.setVisible(false);
		if (UnitexFrame.currentFrame==null) return;
		UnitexFrame.currentFrame.graphicalZone.HCenterAlign();
		UnitexFrame.currentFrame.setModified(true);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
}


class HBottomListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		GraphPreferenceMenu.pref.setVisible(false);
		if (UnitexFrame.currentFrame==null) return;
		UnitexFrame.currentFrame.graphicalZone.HBottomAlign();
		UnitexFrame.currentFrame.setModified(true);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
}


class VLeftListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		GraphPreferenceMenu.pref.setVisible(false);
		if (UnitexFrame.currentFrame==null) return;
		UnitexFrame.currentFrame.graphicalZone.VLeftAlign();
		UnitexFrame.currentFrame.setModified(true);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
}


class VCenterListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		GraphPreferenceMenu.pref.setVisible(false);
		if (UnitexFrame.currentFrame==null) return;
		UnitexFrame.currentFrame.graphicalZone.VCenterAlign();
		UnitexFrame.currentFrame.setModified(true);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
}


class VRightListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		GraphPreferenceMenu.pref.setVisible(false);
		if (UnitexFrame.currentFrame==null) return;
		UnitexFrame.currentFrame.graphicalZone.VRightAlign();
		UnitexFrame.currentFrame.setModified(true);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
}


class CheckBoxListener implements ItemListener {
	public void itemStateChanged(ItemEvent e) {
		JTextField t=(JTextField)GraphPreferenceMenu.pref.nPixels;
		if (GraphPreferenceMenu.pref.checkBox.isSelected()) 
			t.setEditable(true);
		else t.setEditable(false);
	}
}


class OKListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		int n;
		if (UnitexFrame.currentFrame==null) return;
		JCheckBox c=GraphPreferenceMenu.pref.checkBox;
		if (!c.isSelected()) {
			UnitexFrame.currentFrame.setModified(true);
			UnitexFrame.currentFrame.graphicalZone.setGrid(false);
			GraphPreferenceMenu.pref.setVisible(false);
			return;
		}
		if (GraphPreferenceMenu.pref.nPixels.getText().equals("")) {
			return;
		}
		n=toInt(GraphPreferenceMenu.pref.nPixels.getText());
		if (n<10) {
			GraphPreferenceMenu.pref.nPixels.setText("10");
			GraphPreferenceMenu.nPix=10;
			return;
		}
		GraphPreferenceMenu.nPix=n;		
		UnitexFrame.currentFrame.graphicalZone.setGrid(true,n);
		UnitexFrame.currentFrame.setModified(true);
		GraphPreferenceMenu.pref.setVisible(false);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	int toInt(String s) {
		int j=0;
		for (int i=0;i<s.length();i++)
			j=j*10+s.charAt(i)-'0';
		return j;
	}
	public void mouseClicked(MouseEvent e) {}
}


class CANCELListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		GraphPreferenceMenu.pref.setVisible(false);
		e.consume();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
}