 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.* ;

/*
 * This class describes the Inflect window
 *
 */
public class InflectFrame extends JInternalFrame {
	
	public static InflectFrame frame;
	public JPanel panel=new JPanel();
	public JPanel upPanel=new JPanel();
	public JPanel downPanel=new JPanel();
	public JTextField directory=new JTextField("");
	public JButton setDirectory=new JButton("Set...");
	public JButton GO=new JButton("Inflect Dictionary");
	public JButton CANCEL=new JButton("Cancel");
	
	public InflectFrame() {
		super("Inflection",false,true);
		constructPanel();
		setContentPane(panel);
		setBounds(100,100,200,100);
		pack();
		setResizable(false);
		setVisible(false);
		addInternalFrameListener(new InternalFrameListener() {
			public void internalFrameActivated(InternalFrameEvent e) {}
			public void internalFrameClosed(InternalFrameEvent e) {}
			public void internalFrameDeactivated(InternalFrameEvent e) {}
			public void internalFrameDeiconified(InternalFrameEvent e) {}
			public void internalFrameIconified(InternalFrameEvent e) {}
			public void internalFrameOpened(InternalFrameEvent e) {}
			public void internalFrameClosing(InternalFrameEvent e) {
				setVisible(false);
			}
		});
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}


	public static void init() {
		frame=new InflectFrame();
	}
	
	
	public static InflectFrame getFrame() {
		return frame;
	}


	public static void showFrame() {
		frame.setVisible(true);
		frame.directory.setText(Config.getUserCurrentLanguageDir()+"Inflection");
		try { 
		    frame.setSelected(true);
		    frame.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
	}
	
	
	public void constructPanel() {
		panel.setOpaque(true);
		panel.setLayout(new BorderLayout());
		constructUpPanel();
		constructDownPanel();
		panel.add(upPanel,BorderLayout.CENTER);
		panel.add(downPanel,BorderLayout.SOUTH);
	}


	public void constructUpPanel() {
		upPanel.setBorder(new TitledBorder("Directory where inflectional FST2 are stored:"));
		upPanel.setLayout(new BorderLayout());
		directory.setPreferredSize(new Dimension(200,25));
		upPanel.add(directory,BorderLayout.CENTER);
		setDirectory.addMouseListener(new SetDirectoryButtonListener());
		upPanel.add(setDirectory,BorderLayout.EAST);
	}


	public void constructDownPanel() {
		downPanel.setLayout(new GridLayout(1,2));
		CANCEL.addMouseListener(new CancelButtonListener());
		GO.addMouseListener(new GoButtonListener());
		downPanel.add(CANCEL);
		downPanel.add(GO);
	}


class SetDirectoryButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		JFileChooser f=new JFileChooser();
		f.setDialogTitle("Choose your private directory");
		f.setDialogType(JFileChooser.OPEN_DIALOG);
		f.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		f.setCurrentDirectory(new File(Config.getUserCurrentLanguageDir()+"Inflection"));
		int returnVal=f.showOpenDialog(null);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		directory.setText(f.getSelectedFile().getAbsolutePath());
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class GoButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				inflectDELA();
			}
		});
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class CancelButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		setVisible(false);
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


	public void inflectDELA() {
		setVisible(false);
		String command;
		command="\""+Config.getApplicationDir()+"Inflect\" \"";
		command=command+Config.getCurrentDELA()+"\"";
		String tmp=Config.getCurrentDELA();
		int point=tmp.lastIndexOf('.');
		int separator=tmp.lastIndexOf(File.separatorChar);
		if (separator>=point) {
			tmp=tmp+"flx.dic";
		}
		else {
			tmp=tmp.substring(0,point);
			tmp=tmp+"flx.dic";
		}
		command=command+" \""+tmp+"\" ";
		command=command+" \""+directory.getText();
		if (directory.getText().endsWith(File.separator)) {
			command=command.substring(0,command.length()-1);
		}
		command=command+"\"";
		new ProcessInfoFrame(command,false,null);
	}

}