 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.nio.charset.Charset;

/*
 * This class is used to display a lexicon grammar table
 *
 */
public class LexiconGrammarTableFrame extends JInternalFrame {
	JScrollPane scroll;
	JTable table;
	Vector rowData;
	Vector columnNames;
	public static LexiconGrammarTableFrame frame;
	public static String tableName;

	
	public LexiconGrammarTableFrame(String name) {
		super(name,true,true,true,true);
		tableName=name;
		buildVectors(name);
		JPanel top=new JPanel();
		top.setOpaque(true);
		top.setLayout(new BorderLayout());
		table=new JTable(rowData,columnNames);
		table.setFont(Preferences.pref.textFont);
		table.getTableHeader().setReorderingAllowed(false);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		scroll=new JScrollPane(table);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		top.add(scroll,BorderLayout.CENTER);
		setContentPane(top);
		pack();
		setBounds(100,100,800,600);
		setVisible(true);
		frame=this;
		try {
			UnitexFrame.desktop.add(this,UnitexFrame.DOCLAYER);
			setSelected(true);
		} catch (java.beans.PropertyVetoException e) {setVisible(false); frame=null; dispose();}
		addInternalFrameListener(new InternalFrameListener() {
			public void internalFrameActivated(InternalFrameEvent e) {}
			public void internalFrameClosed(InternalFrameEvent e) {}
			public void internalFrameDeactivated(InternalFrameEvent e) {}
			public void internalFrameDeiconified(InternalFrameEvent e) {}
			public void internalFrameIconified(InternalFrameEvent e) {}
			public void internalFrameOpened(InternalFrameEvent e) {}
			public void internalFrameClosing(InternalFrameEvent e) {
				UnitexFrame.mainFrame.compileLexiconGrammar.setEnabled(false);
				UnitexFrame.mainFrame.closeLexiconGrammar.setEnabled(false);
				setVisible(false);
				frame=null;
				dispose();
			}
		});
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}



	public static void closeFrame() {
		if (frame!=null) {
			UnitexFrame.mainFrame.compileLexiconGrammar.setEnabled(false);
			UnitexFrame.mainFrame.closeLexiconGrammar.setEnabled(false);
			frame.setVisible(false);
			frame.dispose();
			frame=null;
		}
	}

	

	public void buildVectors(String name) {
		File f=new File(name);
		if (!f.exists()) {
			JOptionPane.showMessageDialog(null,"Cannot find "+name,"Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (!f.canRead()) {
			JOptionPane.showMessageDialog(null,"Cannot read "+name,"Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (f.length()<=2) {
			JOptionPane.showMessageDialog(null,name+" is empty","Error",JOptionPane.ERROR_MESSAGE); 
			return;
		}
		FileInputStream source;
		try {
			source=new FileInputStream(f);
			char c=read_char(source);
			if (c!=0xFEFF) {
				JOptionPane.showMessageDialog(null,name+" is not a Unicode Little-Endian table file","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			rowData=new Vector();
			// we process the column names first
			String line;
			columnNames=tokenizeToVector(readLine(source));
			// and then the lines
			while (source.available()!=0) {
				rowData.add(tokenizeToVector(readLine(source)));
			}
			source.close();
		} catch (FileNotFoundException e) {return;}
		  catch (IOException e) {return;}
	}


	
	public Vector tokenizeToVector(String s) {
		Vector res=new Vector();
		StringTokenizer st=new StringTokenizer(s,"\t");
		if (st==null) return res;
		while (st.hasMoreElements()) {
			res.add(st.nextToken());
		}
		return res;
	}



	public int toInt(byte b) {
		if (b>0) return b;
		else return 256+b;
	}



	public char read_char(FileInputStream f) {
		byte[] b=new byte[2];
		char c;
		try {
			f.read(b);
		} catch(IOException e) {e.printStackTrace();}
		c=(char)(toInt(b[1])*256+toInt(b[0]));
		if (c==0x0d) {
			read_char(f);
			c='\n';
		}
		return c;
	}



	public String readLine(FileInputStream f) {
		char c;
		String s="";
		while ((c=read_char(f))!='\n') {
			s=s+c;
		}
		return s;
	}
	
}