 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.* ;

/*
 * This class describes the locate pattern window
 *
 */
public class LocateFrame extends JInternalFrame {
	
	public static LocateFrame frame;
	public JPanel panel=new JPanel();
	public JPanel patternPanel=new JPanel();
	public JPanel downPanel=new JPanel();
	public JRadioButton regularExpression=new JRadioButton("Regular expression:",false);
	public JRadioButton graph=new JRadioButton("Graph:",true);
	public JTextField regExp=new JTextField();
	public JTextField graphName=new JTextField();
	public JButton setGraphButton=new JButton("Set");
	public JPanel indexPanel=new JPanel();
	public JPanel outputPanel=new JPanel();
	public JPanel searchLimitationPanel=new JPanel();
	public JButton searchButton=new JButton("SEARCH");
	public JRadioButton shortestMatches=new JRadioButton("Shortest matches",false);
	public JRadioButton longuestMatches=new JRadioButton("Longest matches",true);
	public JRadioButton allMatches=new JRadioButton("All matches",false);
	public JRadioButton ignoreOutputs=new JRadioButton("Are not taken into account",true);
	public JRadioButton mergeOutputs=new JRadioButton("Merge with input text",false);
	public JRadioButton replaceOutputs=new JRadioButton("Replace recognized sequences",false);
	public JRadioButton stopAfterNmatches=new JRadioButton("Stop after ",true);
	public JRadioButton indexAllMatches=new JRadioButton("Index all utterances in text",false);
	public SpecialNumericTextField nMatches=new SpecialNumericTextField("200");
	
	
	public LocateFrame() {
		super("Locate Pattern",false,true);
		constructPanel();
		setContentPane(panel);
		pack();
		setResizable(false);
		setVisible(false);
		addInternalFrameListener(new InternalFrameListener() {
			public void internalFrameActivated(InternalFrameEvent e) {}
			public void internalFrameClosed(InternalFrameEvent e) {}
			public void internalFrameDeactivated(InternalFrameEvent e) {}
			public void internalFrameDeiconified(InternalFrameEvent e) {}
			public void internalFrameIconified(InternalFrameEvent e) {}
			public void internalFrameOpened(InternalFrameEvent e) {}
			public void internalFrameClosing(InternalFrameEvent e) {
				setVisible(false);
			}
		});
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}


	public static void init() {
		frame=new LocateFrame();
	}
	
	
	public static LocateFrame getFrame() {
		return frame;
	}


	public static void showFrame() {
		frame.regExp.setFont(Config.getCurrentTextFont());
		frame.setVisible(true);
		try { 
		    frame.setSelected(true); 
		    frame.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
	}
	
	
	public void constructPanel() {
		panel.setOpaque(true);
		panel.setLayout(new BorderLayout());
		constructPatternPanel();
		constructDownPanel();
		panel.add(patternPanel,BorderLayout.CENTER);
		panel.add(downPanel,BorderLayout.SOUTH);
	}


	public void constructPatternPanel() {
		patternPanel.setBorder(new TitledBorder("Locate pattern in the form of:"));
		setGraphButton.addMouseListener(new SetGraphButtonListener());
		ButtonGroup bg=new ButtonGroup();
		graphName.addMouseListener(new GraphSelectedListener());
		regExp.addMouseListener(new RegularExpressionSelectedListener());
		bg.add(regularExpression);
		bg.add(graph);
		patternPanel.setLayout(new BorderLayout());
		patternPanel.add(regularExpression,BorderLayout.NORTH);
		regExp.setPreferredSize(new Dimension(300,30));
		patternPanel.add(regExp,BorderLayout.CENTER);
		JPanel p=new JPanel();
		p.setLayout(new BorderLayout());
		p.add(graph,BorderLayout.WEST);
		p.add(graphName,BorderLayout.CENTER);
		p.add(setGraphButton,BorderLayout.EAST);
		patternPanel.add(p,BorderLayout.SOUTH);
	}
	
	
	public void constructDownPanel() {
		downPanel.setLayout(new BorderLayout());
		JPanel a=new JPanel();
		JPanel b=new JPanel();
		constructIndexPanel();
		constructOutputPanel();
		constructSearchLimitationPanel();
		a.setLayout(new BorderLayout());
		a.add(indexPanel,BorderLayout.WEST);
		a.add(outputPanel,BorderLayout.CENTER);
		b.setLayout(new BorderLayout());
		b.add(searchLimitationPanel,BorderLayout.WEST);
		searchButton.addMouseListener(new SearchButtonListener());
		b.add(searchButton,BorderLayout.CENTER);
		downPanel.add(a,BorderLayout.CENTER);
		downPanel.add(b,BorderLayout.SOUTH);
	}
	
	
	public void constructIndexPanel() {
		indexPanel.setBorder(new TitledBorder("Index"));
		indexPanel.setLayout(new GridLayout(3,1));
		ButtonGroup bg=new ButtonGroup();
		bg.add(shortestMatches);
		bg.add(longuestMatches);
		bg.add(allMatches);
		indexPanel.add(shortestMatches);
		indexPanel.add(longuestMatches);
		indexPanel.add(allMatches);
	}


	public void constructOutputPanel() {
		outputPanel.setBorder(new TitledBorder("Grammar outputs"));
		outputPanel.setLayout(new GridLayout(3,1));
		ButtonGroup bg=new ButtonGroup();
		bg.add(ignoreOutputs);
		bg.add(mergeOutputs);
		bg.add(replaceOutputs);
		outputPanel.add(ignoreOutputs);
		outputPanel.add(mergeOutputs);
		outputPanel.add(replaceOutputs);
	}


	public void constructSearchLimitationPanel() {
		searchLimitationPanel.setBorder(new TitledBorder("Search limitation"));
		searchLimitationPanel.setLayout(new GridLayout(2,1));
		JPanel p=new JPanel();
		p.setLayout(new BorderLayout());
		p.add(stopAfterNmatches,BorderLayout.WEST);
		p.add(nMatches,BorderLayout.CENTER);
		p.add(new JLabel(" matches"),BorderLayout.EAST);
		ButtonGroup bg=new ButtonGroup();
		bg.add(stopAfterNmatches);
		bg.add(indexAllMatches);
		searchLimitationPanel.add(p);
		searchLimitationPanel.add(indexAllMatches);
	}
	

class SpecialNumericTextField extends JTextField {
	SpecialNumericTextField(String s) {
		super();
		setEditable(true);
		setText(s);
	}
	public Document createDefaultModel() {
		return new SpecialNumericTextDocument();
	}
}


class SpecialNumericTextDocument extends PlainDocument {
	public void insertString(int offs,String s,AttributeSet a) throws BadLocationException {
		int i;
		if (s==null) return;
		char c[]=s.toCharArray();
		for (i=0;i<c.length;i++) {
			if ((c[i]<'0' || c[i]>'9')) return;
		}
		super.insertString(offs,s,a);
	}
}


class SetGraphButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		JFileChooser chooser=new JFileChooser();
		chooser.addChoosableFileFilter(new PersonalFileFilter("fst2","Unicode Compiled Graphs"));
		chooser.addChoosableFileFilter(new PersonalFileFilter("grf","Unicode Graphs"));
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setCurrentDirectory(new File(Config.getCurrentGraphDir()));
		int returnVal=chooser.showOpenDialog(null);
		if (returnVal!=JFileChooser.APPROVE_OPTION) {
			// we return if the user has clicked on CANCEL
			return;
		}
		graphName.setText(chooser.getSelectedFile().getAbsolutePath());
		graph.setSelected(true);
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


class SearchButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		frame.launchLocate();
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



class GraphSelectedListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
		graph.setSelected(true);
	}
}



class RegularExpressionSelectedListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
		regularExpression.setSelected(true);
	}
}


	public void launchLocate() {
		String[] command=new String[10];
		String fst2;
		if (stopAfterNmatches.isSelected() && nMatches.getText().equals("")) {
			JOptionPane.showMessageDialog(null,"Empty search limitation field !","Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (regularExpression.isSelected()) {
			// we need to process a regular expression
			if (regExp.getText().equals("")) {
				JOptionPane.showMessageDialog(null,"Empty regular expression !","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			createRegExpFile(regExp.getText(),Config.getUserCurrentLanguageDir()+"regexp.txt");
			command[0]="\""+Config.getApplicationDir()+"Reg2Grf\" \""+Config.getUserCurrentLanguageDir()+"regexp.txt\"";
			command[1]="\""+Config.getApplicationDir()+"Grf2Fst2\" \""+Config.getUserCurrentLanguageDir()+"regexp.grf\"";
			fst2=Config.getUserCurrentLanguageDir()+"regexp.fst2";
		}
		else {
			// we need to process a graph
			if (graphName.getText().equals("")) {
				JOptionPane.showMessageDialog(null,"You must specify a graph name","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			String n=graphName.getText();
			if (n.substring(n.length()-3,n.length()).equalsIgnoreCase("grf")) {
				// we must compile the grf
				command[1]="\""+Config.getApplicationDir()+"Grf2Fst2\" \""+n+"\"";
				fst2=n.substring(0,n.length()-3);
				fst2=fst2+"fst2";
			}
			else {
				if (!(n.substring(n.length()-4,n.length()).equalsIgnoreCase("fst2"))) {
					// if the extension is nor GRF neither FST2
					JOptionPane.showMessageDialog(null,"Invalid graph name extension !","Error",JOptionPane.ERROR_MESSAGE);
					return;
				}
				fst2=n;
			}
		}
		command[2]="\""+Config.getApplicationDir()+"Locate\" ";
		command[2]=command[2]+"\""+Config.getCurrentSnt()+"\" ";
		command[2]=command[2]+"\""+fst2+"\" ";
		command[2]=command[2]+"\""+Config.getAlphabet()+"\" ";
		if (shortestMatches.isSelected()) command[2]=command[2]+"s ";
		else if (longuestMatches.isSelected()) command[2]=command[2]+"l ";
		else command[2]=command[2]+"a ";
		if (ignoreOutputs.isSelected()) command[2]=command[2]+"i ";
		else if (mergeOutputs.isSelected()) command[2]=command[2]+"m ";
		else command[2]=command[2]+"r ";
		if (stopAfterNmatches.isSelected()) {
			command[2]=command[2]+nMatches.getText()+" ";
		}
		else {
			command[2]=command[2]+"all ";
		}
		if (Config.getCurrentLanguage().equals("Thai") ||
		   	Config.getCurrentLanguage().equals("Korean")) {
			command[2]=command[2]+"-thai ";
		}
		frame.setVisible(false);
		new ProcessInfoFrame(command,true,new LocateDo());
	}
	
	
	public void createRegExpFile(String regExp,String name) {
		File f=new File(name);
		try {
			if (!f.exists()) {
				f.createNewFile();
			}
			BufferedWriter bw=new BufferedWriter(new OutputStreamWriter(new FileOutputStream(f),"UTF-16LE"));
			bw.write('\ufeff');
			bw.write(regExp,0,regExp.length());
			bw.close();
		} catch (IOException e) {}
	}
	
	
	public static int toInt(byte b) {
		if (b>0) return b;
		else return 256+b;
	}


	public static char read_char(FileInputStream f) {
		byte[] b=new byte[2];
		char c;
		try {
			f.read(b);
		} catch(IOException e) {e.printStackTrace();}
		c=(char)(toInt(b[1])*256+toInt(b[0]));
		if (c==0x0d) {
			read_char(f);
			c='\n';
		}
		return c;
	}


	public static String readLine(FileInputStream f) {
		char c;
		String s="";
		while ((c=read_char(f))!='\n') {
			s=s+c;
		}
		return s;
	}


	public static String readInfo(String name) {
		File f=new File(name);
		if (!f.exists()) {return null;}
		if (!f.canRead()) {return null;}
		String res;
		FileInputStream source;
		try {
			source=new FileInputStream(f);
			char c=read_char(source);
			if (c!=0xFEFF) {
				return null;
			}
			res=readLine(source)+"\n";
			res=res+readLine(source)+"\n";
			res=res+readLine(source);
			source.close();
		} catch (FileNotFoundException e) {return null;}
		  catch (IOException e) {return null;}
		  return res;
	}


	public static String readLine(String name) {
		File f=new File(name);
		if (!f.exists()) {return null;}
		if (!f.canRead()) {return null;}
		String res;
		FileInputStream source;
		try {
			source=new FileInputStream(f);
			char c=read_char(source);
			if (c!=0xFEFF) {
				return null;
			}
			res=readLine(source);
			source.close();
		} catch (FileNotFoundException e) {return null;}
		  catch (IOException e) {return null;}
		  return res;
	}

	
class LocateDo extends DoAbstract {
	public LocateDo() {
	}
	public void DO() {
		String res=readInfo(Config.getCurrentSntDir()+"concord.n");
		String res2=readLine(Config.getCurrentSntDir()+"concord.n");
		JOptionPane.showMessageDialog(null,res,"Result Info",JOptionPane.PLAIN_MESSAGE);
		ConcordanceParameterFrame.showFrame(" "+res2);
	}	
}

}
