 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.undo.UndoableEdit;

/*
 * This class describes a JTextArea that can correctly load unicode text
 */
public class MyTextArea extends JTextArea {

    
  public MyTextArea() {
  	super();
  	setLineWrap(true);
  }


  public void load(File file) throws IOException,IllegalArgumentException {
  
   final StringContent content=new StringContent();
   final FileChannel channel=new FileInputStream(file).getChannel();           
   final PlainDocument document=new PlainDocument(content);
   setDocument(document);
   final Timer timer=new Timer(300, null);
   final ByteBuffer buffer=ByteBuffer.allocateDirect(16000);
   buffer.order(ByteOrder.LITTLE_ENDIAN);
   final CharBuffer charBuffer=buffer.asCharBuffer();
    
    // remove unicode header
    buffer.limit(1);
    channel.read(buffer);
    buffer.position(0);
    channel.read(buffer);
    buffer.clear();
  
    // asynchronous update
    timer.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
         try {
       
          if (channel.read(buffer)>0) {
            int bufferPosition=buffer.position();
            if ((bufferPosition & 1)==1)
              throw new InternalError("bad alignement: the text file is corrupted");
          	
            String s=charBuffer.limit(bufferPosition>>1).toString();
            int caretPosition=getCaretPosition();
            try {
              document.insertString(content.length()-1,s,null);
            } catch(BadLocationException e) {
              throw new Error(e);	
            }
            setCaretPosition(caretPosition);
            
            buffer.clear();
          }
          else
            timer.stop();
            
        } catch(IOException e) {
          e.printStackTrace();
          timer.stop();
        }
        }
      });
    timer.setCoalesce(true);
    timer.setRepeats(true);
    timer.start();
  }

}
