 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.awt.*;
import java.io.* ;
import javax.swing.* ;
/*
 * This class describes the graph presentation preferences
 *
 */
class Preferences {
	public Font input;
	public Font output;
	public Font textFont;
	public Color backgroundColor;
	public Color foregroundColor;
	public Color subgraphColor;
	public Color selectedColor;
	public Color commentColor;
	public boolean date;
	public boolean filename;
	public boolean pathname;
	public boolean frame;
	public boolean rightToLeft;
	public int inputSize;
	public int outputSize;
	public int textFontSize;
	public String HtmlFontName;
	public int HtmlFontSize;
	public boolean antialiasing;
	public String htmlViewer=null;
		
	public static Preferences pref=new Preferences();
	public static Preferences temp=new Preferences();
	

	public Preferences() {
		inputSize=10;
		outputSize=12;
		input=new Font("Times New Roman",Font.PLAIN,(int)(inputSize/0.72));
		output=new Font("Times New Roman",Font.BOLD,(int)(outputSize/0.72));
		backgroundColor=Color.white;
		foregroundColor=Color.black;
		subgraphColor=new Color(205,205,205);
		selectedColor=Color.blue;
		commentColor=Color.red;
		date=true;
		filename=true;
		pathname=false;
		frame=true;
		rightToLeft=false;
		textFontSize=10;
		textFont=new Font("Courier new",Font.PLAIN,(int)(textFontSize/0.72));
		HtmlFontName="Courier new";
		HtmlFontSize=3;
		antialiasing=false;
		htmlViewer=null;
	}	
	
	
	public static void setDefaultPreferences() {
		pref.inputSize=10;
		pref.outputSize=12;
		pref.input=new Font("Times New Roman",Font.PLAIN,(int)(pref.inputSize/0.72));
		pref.output=new Font("Times New Roman",Font.BOLD,(int)(pref.inputSize/0.72));
		pref.backgroundColor=Color.white;
		pref.foregroundColor=Color.black;
		pref.subgraphColor=new Color(205,205,205);
		pref.selectedColor=Color.blue;
		pref.commentColor=Color.red;
		pref.date=true;
		pref.filename=true;
		pref.pathname=false;
		pref.frame=true;
		pref.rightToLeft=false;
		pref.textFontSize=10;
		pref.textFont=new Font("Courier new",Font.PLAIN,(int)(pref.textFontSize/0.72));
		pref.HtmlFontName="Courier new";
		pref.HtmlFontSize=3;
		pref.antialiasing=false;
		pref.htmlViewer=null;
		pref.loadConfigFile();
	}
	
	
	public Preferences getClone() {
		Preferences p=new Preferences();
		p.inputSize=inputSize;
		p.outputSize=outputSize;
		p.input=new Font(input.getName(),input.getStyle(),(int)(inputSize/0.72));
		p.output=new Font(output.getName(),output.getStyle(),(int)(outputSize/0.72));
		p.backgroundColor=new Color(backgroundColor.getRGB());
		p.foregroundColor=new Color(foregroundColor.getRGB());
		p.subgraphColor=new Color(subgraphColor.getRGB());
		p.selectedColor=new Color(selectedColor.getRGB());
		p.commentColor=new Color(commentColor.getRGB());
		p.date=date;
		p.filename=filename;
		p.pathname=pathname;
		p.frame=frame;
		p.rightToLeft=rightToLeft;
		p.textFontSize=textFontSize;
		p.textFont=new Font(textFont.getName(),textFont.getStyle(),(int)(textFontSize/0.72));
		p.HtmlFontName=HtmlFontName;
		p.HtmlFontSize=HtmlFontSize;
		p.antialiasing=antialiasing;
		p.htmlViewer=htmlViewer;
		return p;
	}
	
	
	public static String getConcordanceFontName() {
		return pref.HtmlFontName;
	}
	
	
	public static int getConcordanceFontSize() {
		return pref.HtmlFontSize;
	}
	

	public String readTokenOnLine(FileInputStream f) {
		char c;
		String s="";
		try {
			while ((c=read_char(f))!='\n') {
				s=s+c;
			}
		} catch (Exception e) {return null;}
		return s;
	}


	public String readFieldName(FileInputStream f) {
		char c;
		String s="";
		try {
			while ((c=read_char(f))!='=' && c!='\n') {
				s=s+c;
			}
		} catch (IOException e) {return null;}
		return s;
	}
	

	public static int toInt(byte b) {
		if (b>0) return b;
		else return 256+b;
	}


	public static char read_char(FileInputStream f) throws IOException {
		byte[] b=new byte[2];
		char c;
		if (f.available()==0) throw new IOException();
		f.read(b);
		c=(char)(toInt(b[1])*256+toInt(b[0]));
		if (c==0x0d) {
			read_char(f);
			c='\n';
		}
		return c;
	}

	
	public void loadConfigFile() {
		File f=new File(Config.getUserCurrentLanguageDir()+"Config");
		if (!f.exists()) {return;}
		if (!f.canRead()) {return;}
		FileInputStream source;
		try {
			source=new FileInputStream(f);
			char c=read_char(source);
			if (c!=0xFEFF) {
				JOptionPane.showMessageDialog(null,Config.getUserCurrentLanguageDir()+"Config"+" is not a Unicode configuration file","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			
			String field;
			String textFontName="Dialog";
			int textFontStyle=Font.PLAIN;
			String inputFontName="Dialog";
			int inputFontStyle=Font.PLAIN;
			String outputFontName="Dialog";
			int outputFontStyle=Font.PLAIN;
						
			while ((field=readFieldName(source))!=null) {
				if (field.equals("TEXT FONT NAME")) {
					textFontName=readTokenOnLine(source);
				}
				else if (field.equals("TEXT FONT STYLE")) {
					textFontStyle=new Integer(readTokenOnLine(source)).intValue();
				}
				else if (field.equals("TEXT FONT SIZE")) {
					textFontSize=new Integer(readTokenOnLine(source)).intValue();
				}
				else if (field.equals("CONCORDANCE FONT NAME")) {
					HtmlFontName=readTokenOnLine(source);
				}
				else if (field.equals("CONCORDANCE FONT HTML SIZE")) {
					HtmlFontSize=new Integer(readTokenOnLine(source)).intValue();
				}
				else if (field.equals("INPUT FONT NAME")) {
					inputFontName=readTokenOnLine(source);
				}
				else if (field.equals("INPUT FONT STYLE")) {
					inputFontStyle=new Integer(readTokenOnLine(source)).intValue();
				}
				else if (field.equals("INPUT FONT SIZE")) {
					inputSize=new Integer(readTokenOnLine(source)).intValue();
				}
				else if (field.equals("OUTPUT FONT NAME")) {
					outputFontName=readTokenOnLine(source);
				}
				else if (field.equals("OUTPUT FONT STYLE")) {
					outputFontStyle=new Integer(readTokenOnLine(source)).intValue();
				}
				else if (field.equals("OUTPUT FONT SIZE")) {
					outputSize=new Integer(readTokenOnLine(source)).intValue();
				}
				else if (field.equals("DATE")) {
					date=new Boolean(readTokenOnLine(source)).booleanValue();
				}
				else if (field.equals("FILE NAME")) {
					filename=new Boolean(readTokenOnLine(source)).booleanValue();
				}
				else if (field.equals("PATH NAME")) {
					pathname=new Boolean(readTokenOnLine(source)).booleanValue();
				}
				else if (field.equals("FRAME")) {
					frame=new Boolean(readTokenOnLine(source)).booleanValue();
				}
				else if (field.equals("RIGHT TO LEFT")) {
					rightToLeft=new Boolean(readTokenOnLine(source)).booleanValue();
				}
				else if (field.equals("BACKGROUND COLOR")) {
					backgroundColor=new Color(new Integer(readTokenOnLine(source)).intValue());
				}
				else if (field.equals("FOREGROUND COLOR")) {
					foregroundColor=new Color(new Integer(readTokenOnLine(source)).intValue());
				}
				else if (field.equals("AUXILIARY NODES COLOR")) {
					subgraphColor=new Color(new Integer(readTokenOnLine(source)).intValue());
				}
				else if (field.equals("COMMENT NODES COLOR")) {
					commentColor=new Color(new Integer(readTokenOnLine(source)).intValue());
				}
				else if (field.equals("SELECTED NODES COLOR")) {
					selectedColor=new Color(new Integer(readTokenOnLine(source)).intValue());
				}
				else if (field.equals("ANTIALIASING")) {
					antialiasing=new Boolean(readTokenOnLine(source)).booleanValue();
				}
				else if (field.equals("HTML VIEWER")) {
					htmlViewer=readTokenOnLine(source);
				}
				else {
					System.out.println(field+": unknown field");
				}
				
				
			}
			textFont=new Font(textFontName,textFontStyle,(int)(textFontSize/0.72));
			input=new Font(inputFontName,inputFontStyle,(int)(inputSize/0.72));
			output=new Font(outputFontName,outputFontStyle,(int)(outputSize/0.72));
			source.close();
		} catch (FileNotFoundException e) {return;}
		  catch (IOException e) {return;}
	}
	
}