 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.nio.charset.Charset;

/*
 * This class is used to display the text automaton
 *
 */
public class TextAutomatonFrame extends JInternalFrame {
	JTextArea text=new JTextArea();
	SpecialNumericTextField sentence_number=new SpecialNumericTextField(4,"1");
	JScrollPane textScroll;
	public static TextAutomatonFrame frame=null;
	JPanel panel=new JPanel();
	JPanel upPanel=new JPanel();
	JPanel cornerPanel=new JPanel();
	JLabel sentence_count_label=new JLabel (" 0 sentence");
	JButton RESET_SENTENCE_GRAPH=new JButton("Reset Sentence Graph");
	JButton REBUILD_TEXT_AUTOMATON=new JButton("Rebuild FST-Text");
	
	JButton UP=new JButton("\u25B2");
	JButton DOWN=new JButton("\u25BC");
	JScrollPane scroll;
	FstGraphicalZone graphicalZone;
	FstTextField texte;
	boolean modified=false;
	static int sentence_count=0;
	static String text_fst="";
	static String sentence_text="";
	static String sentence_grf="";
	static String sentence_modified="";
	static boolean isAcurrentLoadingThread=false;
	static Process currentLoadingProcess=null;
	
	public TextAutomatonFrame() {
		super("FST-Text",true,true,true,true);
		constructPanel();
		setContentPane(panel);
		pack();
		setBounds(150,150,850,650);
		setVisible(false);
		addInternalFrameListener(new InternalFrameListener() {
			public void internalFrameActivated(InternalFrameEvent e) {}
			public void internalFrameClosed(InternalFrameEvent e) {}
			public void internalFrameDeactivated(InternalFrameEvent e) {}
			public void internalFrameDeiconified(InternalFrameEvent e) {}
			public void internalFrameIconified(InternalFrameEvent e) {}
			public void internalFrameOpened(InternalFrameEvent e) {}
			public void internalFrameClosing(InternalFrameEvent e) {
				try {
					setIcon(true);
				} catch (java.beans.PropertyVetoException e2) {}
			}
		});
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}
	
	
	
	public void constructPanel() {
		panel.setLayout(new BorderLayout());
		constructUpPanel();
		panel.add(upPanel,BorderLayout.NORTH);
		JPanel downPanel=new JPanel();
		downPanel.setLayout(new BorderLayout());
		downPanel.setOpaque(true);
		texte=new FstTextField(25,this);
		graphicalZone=new FstGraphicalZone(1188,840,texte,this);
		graphicalZone.setPreferredSize(new Dimension(1188,840));
		scroll=new JScrollPane(graphicalZone);
		scroll.setOpaque(true);
		scroll.getHorizontalScrollBar().setUnitIncrement(20);
		scroll.getVerticalScrollBar().setUnitIncrement(20);
		scroll.setPreferredSize(new Dimension(1188,840));
		texte.setFont(Preferences.pref.input);
		downPanel.add(texte,BorderLayout.NORTH);
		downPanel.add(scroll,BorderLayout.CENTER);
		panel.add(downPanel,BorderLayout.CENTER);
	}



	public void constructUpPanel() {
		upPanel.setLayout(new BorderLayout());
		text.setFont(Preferences.pref.textFont);
		text.setEditable(false);
		text.setText("");
		text.setLineWrap(true);
		textScroll=new JScrollPane(text);
		textScroll.setOpaque(true);
		textScroll.setPreferredSize(new Dimension(600,100));
		textScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		JPanel tmp=new JPanel();
		tmp.setOpaque(true);
		tmp.setLayout(new BorderLayout());
		tmp.setBorder(new EmptyBorder(2,2,2,2));
		tmp.add(textScroll,BorderLayout.CENTER);
		upPanel.add(tmp,BorderLayout.CENTER);
		constructCornerPanel();
		upPanel.add(cornerPanel,BorderLayout.WEST);
	}



	public void constructCornerPanel() {
		cornerPanel.setBorder(BorderFactory.createRaisedBevelBorder());
		cornerPanel.setLayout(new GridLayout(4,1));
		cornerPanel.add(sentence_count_label);
		JPanel middle=new JPanel();
		middle.setLayout(new BorderLayout());
		middle.add(new JLabel(" Sentence # "),BorderLayout.WEST);
		JPanel right=new JPanel();
		right.setLayout(new BorderLayout());
		right.add(sentence_number,BorderLayout.WEST);
		JPanel tmp=new JPanel();
		tmp.setBorder(new EmptyBorder(0,3,0,3));
		tmp.setLayout(new GridLayout(2,1));
		Font f=new Font("Dialog",Font.PLAIN,5);
		UP.setFont(f);
		DOWN.setFont(f);
		UP.addMouseListener(new UPButtonListener());
		DOWN.addMouseListener(new DOWNButtonListener());
		tmp.add(UP);
		tmp.add(DOWN);
		right.add(tmp,BorderLayout.CENTER);
		middle.add(right,BorderLayout.EAST);
		cornerPanel.add(middle);
		RESET_SENTENCE_GRAPH.addMouseListener(new RESETButtonListener());
		RESET_SENTENCE_GRAPH.setVisible(false);
		cornerPanel.add(RESET_SENTENCE_GRAPH);
		REBUILD_TEXT_AUTOMATON.addMouseListener(new REBUILDButtonListener());
		cornerPanel.add(REBUILD_TEXT_AUTOMATON);
	}
	
	
	
	public static void init() {
		frame=new TextAutomatonFrame();
		UnitexFrame.mainFrame.desktop.add(frame,UnitexFrame.DOCLAYER);
	}

	
	
	public static TextAutomatonFrame getFrame() {
		return frame;
	}



	public static void showFrame() {
		File f=new File(Config.getCurrentSntDir()+"text.fst2");
		if (!f.exists()) {
			// if there is no text FST, we do nothing
			return;
		}
		text_fst=Config.getCurrentSntDir()+"text.fst2";
		sentence_text=Config.getCurrentSntDir()+"cursentence.txt";
		sentence_grf=Config.getCurrentSntDir()+"cursentence.grf";
		sentence_modified=Config.getCurrentSntDir()+"sentence";
		if (frame==null) init();
		sentence_count=readSentenceCount(f);
		String s=" "+frame.sentence_count;
		s=s+" sentence";
		if (sentence_count>1) s=s+"s";
		frame.sentence_count_label.setText(s);
		frame.sentence_number.setText("1");
		loadSentence(1);
		frame.setVisible(true);
		try { 
		    frame.setIcon(true);
		} catch (java.beans.PropertyVetoException e2) {}
	}

	
	public static void close() {
		if (frame==null) return;	
		frame.setVisible(false);
		frame.graphicalZone.is_initialised=false;
		frame.graphicalZone.graphBoxes.removeAllElements();
		frame.graphicalZone.repaint();
		frame.text.setText("");
		try { 
		    frame.setIcon(false);
		} catch (java.beans.PropertyVetoException e2) {}
		System.gc();
	}


	class SpecialNumericTextField extends JTextField implements KeyListener {
		
		SpecialNumericTextField(int c,String s) {
			super(c);
			setText(s);
			addKeyListener(this);
		}
		
		public Document createDefaultModel() {
			return new SpecialNumericTextDocument();
		}
		
		public void keyTyped(KeyEvent e) {}
		
		public void keyPressed(KeyEvent e) {
			if (e.getKeyCode()==10) {
				int n=new Integer(getText()).intValue();
				loadSentence(n);
			}
		}
		
		public void keyReleased(KeyEvent e) {}
	}


	class SpecialNumericTextDocument extends PlainDocument {
		public void insertString(int offs,String s,AttributeSet a) throws BadLocationException {
			int i;
			if (s==null) return;
			char c[]=s.toCharArray();
			for (i=0;i<c.length;i++) {
				if ((c[i]!=10) && (c[i]<'0' || c[i]>'9')) return;
			}
			super.insertString(offs,s,a);
		}
	}


	public void setModified(boolean b) {
		RESET_SENTENCE_GRAPH.setVisible(b);
		if (b) {
			// we save each modification
			GraphIO g=new GraphIO();
			g.boxes=frame.graphicalZone.graphBoxes;
			g.pref=frame.graphicalZone.pref;
			g.width=frame.graphicalZone.Width;
			g.height=frame.graphicalZone.Height;
			g.saveSentenceGraph(sentence_modified+sentence_number.getText()+".grf");
		}
	}


	public static int readSentenceCount(File f) {
		String s="0";
		try {
			FileInputStream br=new FileInputStream(f);
			read_char(br); // skipping 0xFEFF
			s=readLine(br);
			if (s==null || s.equals("")) {
				return 0;
			}
			br.close();
		} catch (IOException e) {}
		return new Integer(s).intValue();
	}
	
	
	public static char read_char(FileInputStream f) {
		byte[] b=new byte[2];
		char c;
		try {
			f.read(b);
		} catch(IOException e) {}
		c=(char)(toInt(b[1])*256+toInt(b[0]));
		if (c==0x0d) {
			read_char(f);
			c='\n';
		}
		return c;
	}
	

	public static int toInt(byte b) {
		if (b>0) return b;
		else return 256+b;
	}


	public static String readLine(FileInputStream f) {
		char c;
		String s="";
		while ((c=read_char(f))!='\n') {
			s=s+c;
		}
		return s;
	}


	
	public static void loadSentenceFromConcordance(int n) {
		if (frame==null || frame.isVisible()==false || frame.isIcon()==true) return;
		if (n<1 || n>sentence_count) return;
		if (loadSentence(n))
			frame.sentence_number.setText(String.valueOf(n));
	}
	


	public static boolean loadSentence(int n) {
		if (n<1 || n>sentence_count) return false;
		final int z=n;
		if (isAcurrentLoadingThread) return false;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new Thread() {
					public void run() {
						isAcurrentLoadingThread=true;
						frame.graphicalZone.is_initialised=false;
						frame.graphicalZone.graphBoxes.removeAllElements();
						frame.graphicalZone.repaint();
						frame.text.setText("");
						String command="\""+Config.getApplicationDir()+"Fst2Grf\" ";
						command=command+"\""+text_fst+"\" ";
						command=command+String.valueOf(z);
						String[] comm=new String[3];
						comm[0]=Config.getApplicationDir()+"Fst2Grf";
						comm[1]=text_fst;
						comm[2]=String.valueOf(z);
						Console.add(command);
						currentLoadingProcess=null;
						try {
							currentLoadingProcess=Runtime.getRuntime().exec(comm);
							BufferedReader inputStream=new BufferedReader(new InputStreamReader(currentLoadingProcess.getInputStream()));
							BufferedReader errorStream=new BufferedReader(new InputStreamReader(currentLoadingProcess.getErrorStream()));
							while (inputStream.readLine()!=null);
						} catch (java.io.IOException e) {return;}
						readSentenceText();
						File f=new File (sentence_modified+String.valueOf(z)+".grf");
						boolean modified=f.exists();
						if (modified) {
							loadSentenceGraph(sentence_modified+String.valueOf(z)+".grf");
							frame.setModified(modified);
						} else loadSentenceGraph(sentence_grf);
						isAcurrentLoadingThread=false;
					}
				}.start();
			}
		});
		return true;
	}


	public void changeAntialiasingValue() {
		graphicalZone.antialiasingValue=!graphicalZone.antialiasingValue;
		graphicalZone.repaint();
	}

	
	public static void readSentenceText() {
		String s="";
		try {
			FileInputStream br=new FileInputStream(new File(sentence_text));
			read_char(br); // skipping 0xFEFF
			s=readLine(br);
			if (s==null || s.equals("")) {
				return;
			}
			frame.text.setFont(Config.getCurrentTextFont());
			frame.text.setText(s);
			br.close();
		} catch (IOException e) {}
	}
	
	
	
	public static void loadSentenceGraph(String name) {
		frame.setModified(false);
		GraphIO g=GraphIO.loadSentenceGraph(name);
		if (g==null) return;
		frame.graphicalZone.is_initialised=false;
		frame.texte.setFont(frame.graphicalZone.pref.input);
		frame.graphicalZone.pref=Preferences.pref.getClone();
		frame.graphicalZone.Width=g.width;
		frame.graphicalZone.Height=g.height;
		frame.graphicalZone.graphBoxes=g.boxes;
		frame.scroll.setPreferredSize(new Dimension(g.width,g.height));
		frame.graphicalZone.setPreferredSize(new Dimension(g.width,g.height));
		frame.graphicalZone.revalidate();
		frame.graphicalZone.repaint();
	}



class RESETButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		File f=new File (sentence_modified+sentence_number.getText()+".grf");
		if (f.exists()) f.delete();
		loadSentence(new Integer(sentence_number.getText()).intValue());
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



class RebuildTextAutomatonDo extends DoAbstract {

	public RebuildTextAutomatonDo() {
	}
	public void DO() {
		Config.deleteFileByName(Config.getCurrentSntDir()+"sentence*.grf");
		TextAutomatonFrame.showFrame();
		try {
			TextAutomatonFrame.getFrame().setIcon(false);
			TextAutomatonFrame.getFrame().setSelected(true);
		} catch (java.beans.PropertyVetoException e) {}
	}	
}


class REBUILDButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run(){
				TextAutomatonFrame.close();
				String command[]=new String[1];
				command[0]="\""+Config.getApplicationDir()+"MergeTextAutomaton\" \""+Config.getCurrentSntDir();
				command[0]=command[0]+File.separator+"text.fst2";
				TextAutomatonFrame.close();
				new ProcessInfoFrame(command,true,new RebuildTextAutomatonDo());
			}
		});

	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}



class UPButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		int n=new Integer(sentence_number.getText()).intValue();
		if (n<sentence_count) {
			if (loadSentence(n+1)) {
				n++;
				sentence_number.setText(String.valueOf(n));
			}
		}
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
}



class DOWNButtonListener implements MouseListener {
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {
		int n=new Integer(sentence_number.getText()).intValue();
		if (n>1) {
			if (loadSentence(n-1)) {
				n--;
				sentence_number.setText(String.valueOf(n));
			}
		}
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
	}
}


}
