 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.io.*;
import java.awt.*;
import java.awt.Toolkit;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.swing.text.TextAction.*;

/*
 * This class describes the text field used to get the box text
 *
 */
public class TextField extends JTextField implements KeyListener {  

	public GraphFrame parent;
	public boolean modified=false;
	static public String leftContext;
	static public String rightContext;
	public SpecialPaste specialPaste;
	public SpecialCopy specialCopy;
	public SelectAll selectAll;
	public Cut cut;
	public Preferences preferences;
	public Presentation presentation;
	public Open OPEN;
	public Save SAVE;
	public Clipboard clip;
	
	public TextField(int n,GraphFrame p) {
		super(n);
		setEditable(false);
		modified=false;
		parent=p;
		leftContext="";
		rightContext="";
		clip=Toolkit.getDefaultToolkit().getSystemClipboard();
		setDisabledTextColor(Color.white);
		setBackground(Color.white);
		Keymap k=getKeymap();
		k=addKeymap("textfield-keymap",k);
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('l',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('L',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('k',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('K',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('o',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('O',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('s',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('S',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('p',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('P',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('m',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('M',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('r',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('R',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('x',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('X',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('c',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('C',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('v',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('V',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('a',Event.CTRL_MASK ));
		k.removeKeyStrokeBinding(KeyStroke.getKeyStroke('A',Event.CTRL_MASK ));
		specialPaste=new SpecialPaste("special-paste");
		specialCopy=new SpecialCopy("special-copy");
		presentation=new Presentation("presentation");
		selectAll=new SelectAll("select all");
		cut=new Cut("cut");
		OPEN=new Open("open");
		SAVE=new Save("save");
				
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('v',Event.CTRL_MASK ),specialPaste);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('V',Event.CTRL_MASK ),specialPaste);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('c',Event.CTRL_MASK ),specialCopy);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('C',Event.CTRL_MASK ),specialCopy);

		k.addActionForKeyStroke(KeyStroke.getKeyStroke('r',Event.CTRL_MASK ),presentation);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('R',Event.CTRL_MASK ),presentation);

		k.addActionForKeyStroke(KeyStroke.getKeyStroke('o',Event.CTRL_MASK ),OPEN);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('O',Event.CTRL_MASK ),OPEN);

		k.addActionForKeyStroke(KeyStroke.getKeyStroke('s',Event.CTRL_MASK ),SAVE);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('S',Event.CTRL_MASK ),SAVE);

		k.addActionForKeyStroke(KeyStroke.getKeyStroke('a',Event.CTRL_MASK ),selectAll);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('A',Event.CTRL_MASK ),selectAll);

		k.addActionForKeyStroke(KeyStroke.getKeyStroke('x',Event.CTRL_MASK ),cut);
		k.addActionForKeyStroke(KeyStroke.getKeyStroke('X',Event.CTRL_MASK ),cut);

		this.setKeymap(k);
				
		addKeyListener(this);
	}


	public GraphFrame parent() {
		return UnitexFrame.currentFrame;
	}



	class SpecialCopy extends TextAction implements ClipboardOwner {
		public SpecialCopy(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			if (parent().graphicalZone.selectedBoxes.isEmpty() ||
			    parent().graphicalZone.selectedBoxes.size()<2) {
				// is there is no or one box selected, we copy normally
				copy();
				UnitexFrame.clip.setContents(null,this);
				return;
			}
			UnitexFrame.clip.setContents(new MultipleBoxesSelection(new MultipleSelection(parent().graphicalZone.selectedBoxes)),this);
		}
		
		public void lostOwnership(Clipboard c,Transferable d) {
		}
	}



	class SelectAll extends TextAction implements ClipboardOwner {
		public SelectAll(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			if (!parent().graphicalZone.selectedBoxes.isEmpty() &&
			    parent().graphicalZone.selectedBoxes.size()==1)
				selectAll();
			else parent().graphicalZone.selectAllBoxes();
		}
		
		public void lostOwnership(Clipboard c,Transferable d) {
		}
	}


	class Cut extends TextAction implements ClipboardOwner {
		public Cut(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			if (!parent().graphicalZone.selectedBoxes.isEmpty() &&
			    parent().graphicalZone.selectedBoxes.size()==1) {
				cut();
			}
			else {
				UnitexFrame.clip.setContents(new MultipleBoxesSelection(new MultipleSelection(parent().graphicalZone.selectedBoxes)),this);
				parent().graphicalZone.removeSelected();
				setText("");
			}
		}


		public void lostOwnership(Clipboard c,Transferable d) {
		}
	}



	class SpecialPaste extends TextAction {
		public SpecialPaste(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			Transferable data;
			MultipleSelection res=null;
			data=UnitexFrame.clip.getContents(this);
			try {
				if (data!=null) res=(MultipleSelection)data.getTransferData(new DataFlavor("unitex/boxes","Unitex dataflavor"));
			} catch (UnsupportedFlavorException e2) {}
			  catch (IOException e2) {}
			if (res==null || TextField.this.modified==true) {
				// if there is no boxes to copy we do a simple paste
				paste();
				return;
			}
			res.n++;
			parent().graphicalZone.copySelection(res);
		}
	}



	class Presentation extends TextAction {
		public Presentation(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			new GraphPresentationMenu();
		}
	}


	class Open extends TextAction {
		public Open(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			UnitexFrame.mainFrame.openGraph();
		}
	}


	class Save extends TextAction {
		public Save(String s) {
			super(s);
		}

		public void actionPerformed(ActionEvent e) {
			UnitexFrame.mainFrame.saveGraph(UnitexFrame.currentFrame);
		}
	}


	public void initText(String s) {
		modified=false;
		setEditable(true);
   		setText(s);
   		requestFocus();
		getCaret().setVisible(true);
		selectAll();
	}
	
	
	public Document createDefaultModel() {
		return new ListDocument();
	}

 
	public boolean validateTextField() {
		if (!hasChangedTextField()) return false;
		if (isGoodText(getText())) {
			parent().setModified(true);
        	parent().graphicalZone.setTextForSelected(getText());
    		parent().graphicalZone.unSelectAllBoxes();
    		setText("");
	    	parent().graphicalZone.repaint();
	    	setEditable(false);
	    	return true;
    	}
    	return false;
	}


	public void keyTyped(KeyEvent e) {
	}


	public void keyPressed(KeyEvent e) {
		if (e.isControlDown() || e.isAltDown()) {
			// if the control key or alt key is pressed, we do nothing: the event we be caught by the ActionListeners
			return;
		}
		if (e.getKeyCode()==10) validateTextField();
		modified=true;
	}


	public void keyReleased(KeyEvent e) {
	}


	public boolean hasChangedTextField() {
		return modified;
	}


	public int test_transduction(char s[],int i)
	{
  	int compteur;

  	if(s[i]!='/') return 0;
  	i--;
  		compteur=0;
  	while(i>=0 && s[i]=='\\')
    	{
      	compteur++;
      	i--;
    	}
  	if((compteur%2)!=0) return 0;
  	return 1;
	}


	public boolean tokenize(String s) {
		int L=s.length(),i=0;
		String tmp;
		char ligne[]=new char[10000];
		ligne=s.toCharArray();
		if (ligne[0]=='+') {
  			JOptionPane.showMessageDialog(null,"Unexpected \"+\" as first character of the line","Error",JOptionPane.ERROR_MESSAGE); 
			return false;
		}
		if (ligne[L-1]=='+') {
  			JOptionPane.showMessageDialog(null,"Unexpected \"+\" as last character of the line","Error",JOptionPane.ERROR_MESSAGE); 
			return false;
		}
		while (i<L) {
		  	tmp="";
		  	if (ligne[i]==':') {
		    	// case of a sub graph call
		    	i++;
		    	while ((i<L) && (ligne[i]!='+')) {
		    		if (ligne[i]=='\\') {
		    	  		tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	  		if (i>=L) {
		    	  			JOptionPane.showMessageDialog(null,"Unexpected \"\\\" at end of line","Error",JOptionPane.ERROR_MESSAGE); 
							return false;
		    	  		}
		    		}
		    		tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	}
		    	if (tmp.length()==0) {
    	  			JOptionPane.showMessageDialog(null,"Missing graph name after \":\"","Error",JOptionPane.ERROR_MESSAGE); 
					return false;
		    	}
		    	i++;
		    } else {
		        // all other cases
		        if (ligne[i]=='+') {
		        	// if we find a + just after a + it is an error
		        	JOptionPane.showMessageDialog(null,"Empty line error: \"++\"","Error",JOptionPane.ERROR_MESSAGE); 
					return false;
		        }
		    	while ((i<L) && (ligne[i]!='+')) {
		    		if (ligne[i]=='"') {
		    			// case of a quote expression
		    			tmp=tmp.concat(String.valueOf(ligne[i++]));
		    			while ((i<L) && ligne[i]!='"') {
		    				if (ligne[i]=='\\') {
		    	  				tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	  				if (i>=L) {
				    	  			JOptionPane.showMessageDialog(null,"Unexpected \"\\\" at end of line","Error",JOptionPane.ERROR_MESSAGE); 
									return false;
		    	  				}
		    				}
		    				tmp=tmp.concat(String.valueOf(ligne[i++]));	
		    			}
		    			if (i>=L) {
		    	  			JOptionPane.showMessageDialog(null,"No closing \"","Error",JOptionPane.ERROR_MESSAGE); 							return false;
		    	  		}
		    	  		tmp=tmp.concat(String.valueOf(ligne[i++]));	
		    		}
		    		else if (ligne[i]=='<') {
		    			// case of a <...> expression
		    			tmp=tmp.concat(String.valueOf(ligne[i++]));
		    			while ((i<L) && ligne[i]!='>') {
		    				if (ligne[i]=='\\') {
		    	  				tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	  				if (i>=L) {
				    	  			JOptionPane.showMessageDialog(null,"Unexpected \"\\\" at end of line","Error",JOptionPane.ERROR_MESSAGE); 
									return false;
		    	  				}
		    				}
		    				tmp=tmp.concat(String.valueOf(ligne[i++]));	
		    			}
		    			if (i>=L) {
		    	  			JOptionPane.showMessageDialog(null,"No closing \">\"","Error",JOptionPane.ERROR_MESSAGE); 
							return false;
		    	  		}
		    	  		tmp=tmp.concat(String.valueOf(ligne[i++]));	
		    		}
		    		else {
		    			if (ligne[i]=='\\') {
		    	  			tmp=tmp.concat(String.valueOf(ligne[i++]));
		    	  			if (i>=L) {
			    	  			JOptionPane.showMessageDialog(null,"Unexpected \"\\\" at end of line","Error",JOptionPane.ERROR_MESSAGE); 
								return false;
		    	  			}
		    			}
			    		tmp=tmp.concat(String.valueOf(ligne[i++]));
			    	}
		    	}
		    	i++;
		    }
		}
		return true;		
	}

	
	public boolean isGoodText(String s) {
		if (s.equals("")) return true;
		char ligne[]=new char[10000];
		String tmp="";
		int i,j,L;
		
		ligne=s.toCharArray();
		i=0;
		L=s.length();
		if (L==2 && ligne[0]=='$' && (ligne[1]=='(' || ligne[1]==')')) {
			JOptionPane.showMessageDialog(null,"You must indicate a variable name between $ and ( or )","Error",JOptionPane.ERROR_MESSAGE); 
			return false;
		}
		while((i!=L) && (test_transduction(ligne,i) == 0))
			tmp=tmp.concat(String.valueOf(ligne[i++]));
		if ((i!=L) && (i==0)) {
			JOptionPane.showMessageDialog(null,"Empty text before \"/\"","Error",JOptionPane.ERROR_MESSAGE); 
			return false;
   		}
   		if (L>2 && ligne[0]=='$' && (ligne[L-1]=='(' || ligne[L-1]==')') && s.lastIndexOf('+')==-1) {
   			// case of a variable start $a( or end $a)
   			for (int k=1;k<L-1;k++)
   				if (ligne[k]!='_' && !(ligne[k]>='0' && ligne[k]<='9') && !(ligne[k]>='a' && ligne[k]<='z') 
   				    && !(ligne[k]>='A' && ligne[k]<='Z')) {
   				    	JOptionPane.showMessageDialog(null,"A variable name can only contain the following characters:\nA..Z,a..z,0..9 and the underscore '_'","Error",JOptionPane.ERROR_MESSAGE); 
						return false;
   				}
   			return true;
   		}
   		if (tokenize(tmp)) return true;
   		return false;
	}

}
