 /*
  * Unitex 
  *
  * Copyright (C) 2001-2002 Universit de Marne-la-Valle <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
import java.util.*;


/*
 * This class describes the UNDO actions container
 *
 */
public class UNDOContainer {

	public static final int PREFERENCE_CHANGE=1;
	public static final int POSITION_CHANGE=2;
	public static final int TEXT_CHANGE=3;
	public static final int BOX_SUPPRESSION=4;
	public static final int BOX_CREATION=5;

	public UNDOAction action;
	public GraphFrame frame;


	public UNDOContainer(GraphFrame f) {
		frame=f;
	}


	public void undo() {
		if (action!=null) {
			action.undo();
			action=null;
		}
	}


	public void addUNDOPreferences(Preferences p) {
		action=new UNDOPreferences(frame,p);
		frame.modified=true;
	}


	public void addUNDOPosition(Vector v) {
		action=new UNDOPosition(frame,v);
		frame.modified=true;
	}


	public void addUNDOText(Vector v) {
		action=new UNDOText(frame,v);
		frame.modified=true;
	}


	public void addUNDOCreation(Vector v) {
		action=new UNDOCreation(frame,v);
		frame.modified=true;
	}


	public void addUNDOSelection(Vector v) {
		action=new UNDOSelection(frame,v);
		frame.modified=true;
	}


	public void addUNDOSuppression(Vector v) {
		action=new UNDOSuppression(frame,v);
		frame.modified=true;
	}


}